###########################################
# CreateDEMSounds.py - Creates DEM from 1 Deg USGS DEM data - on local hard drive
# Draft version Date: November 9th, 2011
# Script Written By:  Kirk Sherrill NPS-NRPC-I&M GIS Program
# 
# Abstract:  By defined area of interest and associated DEMs - Uncompresses, mosaics and clips adf files from the I&M hard drive .adf copies of 1 degree USGS DEMs. 
# An extent table (.csv) per Area of Interest is needed with the ULLAT, ULLON, LRLAT, and LRLON.  See Example below which will create a DEM:
# with an extent from 41N -106W to 39N -104W.
# If Clipping to a specific boundary (i.e. Park Boundary, etc.) within the aoa, use the aoa variable and the corresponding park variable
# to define the clip file.  Will reproject to a defined projection such Albers.  
#
# Park, ULLAT, ULLON, LRLAT, LRLON
# ROMO, 41, 106, 40, 105
##########################################

###################################################
# Start of Parameters requiring set up.
###################################################

## Required Data Layers
workspace = "E:\Sounds2012\DEM_Sounds2012_Correct.gdb\\"                  ## Setting output workspace variable
extentList = "E:\\Sounds2012\\Extent_10Mileaoa_Modified.csv"      ##List of Park Extents per defined aoa
RawDEM = "I:\\USGS_NED\NED_HIGH_RES\\conus_13_arc_second_1_degree_tiles\\"       ##Workspace with the raw DEM files
aoa = "E:\\Sounds2012\\aoa_CONUS.gdb\\buf_10mile\\"         ##Location holding the aoa files in the desired output projection
aoaSuffix = "10mile"        ##Output DEM Suffix
clip = "Yes"    ## ('Yes'|'No') If clipping to a specific boundary with in the aoa (i.e. Park Boundary, Mgmt Unit etc is desired) 
workspaceDir = "E:\\Sounds2012\\workspace\\"                    ##workspace directory for temporary files
projDem = "E:\\Sounds2012\\Road_Density_1mile.gdb\\Rdd_All_ACAD001_ACAD"    ##DEM defining the desired Albers output projection
projGCS = "E:\\Sounds2012\\DEM_Sounds2012.gdb\\BRCA022_BRCA_dem"        ##Raster layer with a GCS/NAD 83 projection

ziptool = "C:\\Temp\\7-zip\\7z.exe"     ## Path to the 7zip executable
logFileWorkspace = "E:\\Sounds2012\\workspace\\" 
logFileName = logFileWorkspace + "AA_Create_DEM_SoundSites.txt"  ## Log File for the Create_DEM_Arc10.py script

###################################################
# End of Parameters requiring set up.
###################################################

############################################ 
## Below are paths which are hard coded

# Import system modules
import sys, string, os, arcgisscripting, math, arcpy, glob
from arcpy import env
from arcpy.sa import *


# Create the Geoprocessor object
gp = arcgisscripting.create(9.3)        #Is this needed any more?
# Check out any necessary licenses
arcpy.CheckOutExtension("Spatial")
# Set overwrite files to True 
arcpy.geoprocessing.env.overwriteOutput = True
# Set geoprocessing work space
arcpy.geoprocessing.env.workspace = workspace + "Source\\" 

gp.AddToolbox("C:/arcgis/Desktop10.0/ArcToolbox/Toolboxes/Spatial Analyst Tools.tbx")
gp.AddToolbox("C:/arcgis/Desktop10.0/ArcToolbox/Toolboxes/Conversion Tools.tbx")
gp.AddToolbox("C:/arcgis/Desktop10.0/ArcToolbox/Toolboxes/Data Management Tools.tbx")



##################################
# Checking for working directories
##################################

if os.path.exists(workspace):
    pass
else:
    os.makedirs(workspace)


def timeFun():          #Function to Grab Time
    from datetime import datetime
    b=datetime.now()
    messageTime = b.isoformat()
    return messageTime

if os.path.exists(logFileName):
    pass
else:
    logFile = open(logFileName,"w")
    logFile.close()

#################################################
##Beginning routine to run indivual Fuzzy Viewsheds per development point
#################################################
try:
    
            

    DEMFolders = glob.glob(RawDEM + "dem*")      #Grab the list of folders containing the DEM's needing to be extracted
    '''
    arcpy.geoprocessing.env.workspace = workspace
    rasterList = arcpy.ListRasters("*", "GRID")

    for raster in rasterList:
        logFile = open(logFileName, "a")
        scriptMsg = "Successfully Processed Mosaic for " + raster
        logFile.write(scriptMsg + "\n")
        logFile.close()
    '''

    #Process Each park in the extentList csv file
    readFile = open(extentList, "r")
    lineCount = 1
    for line in readFile:
        parkZipFiles = []
        split = line.split(",")
        park = str(split[0])
        clipFile = aoa + park + "_" + aoaSuffix         #Final Output clipped file
        outDem = workspace + park + "_dem"       #Final DEM
        outDemName = park + "_dem"
        
        # Open log file to get a current list of previously-processed Parks
        lf_read = ""
        
        logFile = open(logFileName, "r")
        lf_read = logFile.read()
        logFile.close()
        testLine = "Successfully Processed Mosaic for " + outDemName   #Checking logfile for this syntax - if there then already processed
        if testLine in lf_read:
            
            messageTime = timeFun()
            logFile = open(logFileName, "a")
            scriptMsg = "Skipping - " + outDemName + "has been previously processed" + messageTime
            logFile.write(scriptMsg + "\n")
            logFile.close()
            pass
        else:
            
            logFile = open(logFileName, "a")       
            if lineCount == 1:
                pass
                lineCount = lineCount + 1
            else:    
                ULLat = int(split[1])
                ULLon = int(split[2])
                LRLat = int(split[3])
                LRLon = int(split[4])
                
                
                toProcess = []
                folderCount = 1
                for folder in DEMFolders:
                    foldSplit = folder.split("\\")
                    lenFoldSplit = len(foldSplit)
                    name = foldSplit[lenFoldSplit - 1]
                    lenName = len(str(name))
                    
                    if lenName == 8 or lenName == 7:
                        if lenName == 8:

                            tLon = name[3:6]
                            if str(tLon.isdigit()) =='True':            #Check for input lon is numeric else break from loop
                                foldLon = int(name[3:6])
                            else:
                                continue                #If not numric continue to next folder

                            tLat = name[-2:]
                            if str(tLat.isdigit()) =='True':            #Check for input lat is numeric else break from loop
                                foldLat = int(name[-2:])
                            else:
                                continue
                            
                        else:

                            tLon = name[3:5]
                            if str(tLon.isdigit()) =='True':            #Check for input lon is numeric else break from loop
                                foldLon = int(name[3:5])    
                            else:
                                continue

                            tLat = name[-2:]
                            if str(tLat.isdigit()) =='True':        #Check for input lat is numeric else break from loop
                                foldLat = int(name[-2:])
                            else:
                                continue
                 
                    else:       #Not in 7 or 8 character format
                        
                        continue
                        folderCount = folderCount + 1
                   
                    if foldLat <= ULLat and foldLat >= LRLat:        #Testing latitude range
                        if foldLon >= LRLon and foldLon <= ULLon:  #Testing longitude range
                            
                            parkZipFiles.append(folder)
                            test = glob.glob(folder + "\\*.adf")
                            toProcess.append(folder)
                            if len(test)== 0:        #Not previously unzipped
                                zipfile = folder + "\\" + name + ".zip"
                                zipcommand = (ziptool + " e " + zipfile + " -o" + folder) #extracting gzip file after intial untar
                                os.system(zipcommand)          #Syntaxed out
                                print folder

                            else:   #Prevously unzipped
                                continue
                        else:       #Not in defined longitude range
                            continue                            
                    
                    else:  #Not in defined latitude range  
                        continue
                
                    folderCount = folderCount + 1 
                   

                ##########################################
                # Per defined set of tiles mosaic per park
                ##########################################
                workDem = park.lower() + "_work.img"            #Kirk this is not dynamic change
                workDem2 = park.lower() + "_work2.img"
                
                workspaceWork = workspaceDir                
                if os.path.exists(workspaceWork):
                    pass
                else:
                    os.mkdir(workspaceWork)

                
                syntax = ""
                lenProcess = len(toProcess)
                pc = 1
                for dem in toProcess:
                    if pc <> lenProcess:
                        syntax = syntax + dem + ";"
                        pc = pc + 1
                    else:
                        syntax = syntax + dem
                        
                if clip.lower()== "no":
                    name = os.path.basename(outDem)
                                    
                    # Process: MosaicToNew
                    arcpy.MosaicToNewRaster_management(syntax, workspace, name, "", "32_BIT_FLOAT", "","1", "LAST", "FIRST")

                else:       #When clipped to aoa file
                
                    # Process: MosaicToNew
                    arcpy.MosaicToNewRaster_management(syntax, workspaceWork, workDem, "", "32_BIT_FLOAT", "","1", "LAST", "FIRST")
                    describeGCS = arcpy.Describe(projGCS)
                    coordinateGCS = describeGCS.spatialReference
                    workDemFull = workspaceWork + "\\" + workDem
                    arcpy.DefineProjection_management(workDemFull, coordinateGCS)

                    describeDem = arcpy.Describe(projDem)   #Describe the aoa DEM
                    coordinateSystem = describeDem.spatialReference    
                    
                    outProjected = workspaceWork + "\\" + workDem2                    
                    
                    arcpy.ProjectRaster_management (workDemFull, outProjected, coordinateSystem, "CUBIC", 10, "", "")
                    
                    arcpy.geoprocessing.env.workspace = workspace
                    # Execute ExtractByMask
                    outExtractByMask = ExtractByMask(outProjected, clipFile)

                    # Save the output 
                    outExtractByMask.save(outDem)
                    # Execute Delete
                    arcpy.Delete_management(workDemFull)
                    arcpy.Delete_management(outProjected)

                            
                messageTime = timeFun()            
                scriptMsg = "Successfully Processed Mosaic for " + outDemName + "_" + messageTime 
                
                logFile.write(scriptMsg + "\n")
                lineCount = lineCount + 1
                logFile.close()    
            
            
except:
    messageTime = timeFun()
    scriptMsg = "Exiting CreateDEMSounds.py Error" 
    print "Exiting CreateDEMSounds.py Exited\nSee log file " + logFileName + " for more details - " + messageTime
    logFile = open(logFileName, "a")
    logFile.write(scriptMsg + "\n")
    logFile.close()
    
finally:
    messageTime = timeFun()
    scriptMsg = "Create CreateDEMSounds.py Completed << " + messageTime
    logFile = open(logFileName, "a")
    logFile.write(scriptMsg + "\n")
    logFile.close()
    