###########################################
# CreateGdb_CopyTable_NHDPlus.py 
# Draft version Date: April 18th, 2012
# Script Written By:  Kirk Sherrill NPS-NRPC-I&M GIS Program
# 
# Abstract:  
#  Creates geodatbases by NHDPlus region and populates with the respective
#  NHDFlowline stream data and Strahler Stream Order (SO) and Strahler Calculator
#  (SC) stream order table. 
#
##########################################

###################################################
# Start of Parameters requiring set up.
###################################################

## Required Data Layers
workspace = "E:\\Sounds2012\\workspace\\"            ##Scratch workspace
outWorkspace = "E:\\Sounds2012\\NHD_Plus\\"        ##output Workspace
processList = "E:\\Sounds2012\\NHD_Plus\\Table\\NHDOrderListAllRegions.csv"           ##List of NHDPlus regions to process

logFileName = workspace + "CreateGdb_CopyTable_NHDPlus.txt"    ##Logfile Name
###################################################
# End of Parameters requiring set up.
###################################################

# Import system modules
import sys, string, os, arcgisscripting, math, arcpy, glob
from arcpy import env
from arcpy.sa import *

# Check out any necessary licenses
arcpy.CheckOutExtension("Spatial")
# Set overwrite files to True 
arcpy.geoprocessing.env.overwriteOutput = True
# Set geoprocessing work space
arcpy.geoprocessing.env.workspace = workspace

if os.path.exists(logFileName):
    pass
else:
    logFile = open(logFileName, "w")
    logFile.close()


def timeFun():          #Function to Grab Time
    from datetime import datetime
    b=datetime.now()
    messageTime = b.isoformat()
    return messageTime

#################################################
##Beginning routine to run indivual Fuzzy Viewsheds per development point
#################################################
try:
        
    readFile = open(processList, "r")

    lineCount = 1
    for line in readFile:               #Loop Per Park
        if lineCount == 1:
            lineCount = lineCount + 1
            pass
        else:
            split = line.split(",")
            
            region = str(split[0])
            streamFile = str(split[1])
            orderTable = str(split[2])
            outWorkspace2 =  outWorkspace       #Output File
            outGdbFull = outWorkspace2 + region + ".gdb"
            outFile = outGdbFull + "\\nhdflowline" 


            lf_read = ""
            logFile = open(logFileName, "r")
            lf_read = logFile.read()
            logFile.close()

            testLine = "Completed " + outFile

            if testLine in lf_read:
                
                messageTime = timeFun()
                logFile = open(logFileName, "a")
                scriptMsg = "Skipping Point - " + outFile + " - has been previously processed" + messageTime
                logFile.write(scriptMsg + "\n")
                logFile.close()
                continue

           
            else: 

                #Create GDB per Park
                arcpy.CreateFileGDB_management(outWorkspace2, region, "CURRENT")
                
                arcpy.FeatureClassToGeodatabase_conversion(streamFile,outGdbFull)
                arcpy.TableToGeodatabase_conversion(orderTable, outGdbFull)
                base = os.path.basename(orderTable)
                base1 = base.split(".")
                newName = str(base1[0])
                gdbTable = outGdbFull + "\\" + newName
                field1 = "Strahler"
                field2 = "StrahCal"

                arcpy.AddField_management(outFile, field1, "DOUBLE", "", "", "", "", "NULLABLE", "")
                arcpy.AddField_management(outFile, field2, "DOUBLE", "", "", "", "", "NULLABLE", "")
                
                
                messageTime = timeFun()
                logFile = open(logFileName, "a")
                
                scriptMsg = "Completed " + outFile + " - " + messageTime
                logFile.write(scriptMsg + "\n")
                logFile.close()


                lineCount = lineCount + 1
            
except:
    messageTime = timeFun()
    scriptMsg = "SelectByStreamOrder.py Error" 
    print "SelectByStreamOrder.py Exited\nSee log file " + logFileName + " for more details - " + messageTime
    
    
finally:
    messageTime = timeFun()
    scriptMsg = "Select By Attribute " + " << " + messageTime
   
    