###########################################
# Create_Slope.py 
# Draft version Date: May 17, 2011
# Script Written By:  Kirk Sherrill NPS-NRPC-I&M GIS Program
# 
# Abstract:  Creates slope for a defined AOA extent from a Digital Elevation model. 
##########################################

###################################################
# Start of Parameters requiring set up.
###################################################

## Required Data Layers
workspace = "D:\\SolarPEIS\\Sounds\\workspace\\"            ##root workspace
inputWorkspace = "D:\\SolarPEIS\\Sounds\\DEM_10m.gdb\\"  ##Input data and or workspace to be clipped                                    
aoaFile = "D:\\SolarPEIS\\Sounds\\data\\AOI_CONUS_10.gdb\\"     ##Location with mask/clip files
outGdb = "D:\\SolarPEIS\\Sounds\\Slope_Sites.gdb"             ##Output GDB
parkList = "D:\\SolarPEIS\\Sounds\\CONUS_SiteID.csv"           ##List of Parks to Process
prefix = "Slope"    ##Prefix name for clipped output        

logFileName = workspace + "AA_Create_Slope.txt"         ##LogFile Name


###################################################
# End of Parameters requiring set up.
###################################################

# Import system modules
import sys, string, os, arcgisscripting, math, arcpy, glob
from arcpy import env
from arcpy.sa import *

# Check out any necessary licenses
arcpy.CheckOutExtension("Spatial")
# Set overwrite files to True 
arcpy.geoprocessing.env.overwriteOutput = True
# Set geoprocessing work space
arcpy.geoprocessing.env.workspace = workspace



def timeFun():          #Function to Grab Time
    from datetime import datetime
    b=datetime.now()
    messageTime = b.isoformat()
    return messageTime

#################################################
##Beginning routine to run indivual Fuzzy Viewsheds per development point
#################################################
try:
        
    readFile = open(parkList, "r")
    
    lineCount = 1
    for line in readFile:               #Loop Per Park
        if lineCount == 1:
            lineCount = lineCount + 1
            pass
        else:
            split = line.split(",")
            park = str(split[0])
            clipFile = aoaFile + "Park_" + park         #Park Specific file to do the clipping/Masking
  
            env.workspace = inputWorkspace
            dataFiles = arcpy.ListRasters("*", "GRID") #Grab the list of folders containing the files needing to be extracted
           
            fileCount = 1
            for file in dataFiles:          #Data Source Files to be clipped/extracted
                
                            
                filePath = file.split("\\")
                t = len(filePath)
                name = filePath[t-1]
                nameSplit = name.split("_")
                prefix = str(nameSplit[0])
                
                
                outFile = outGdb + park + "_slope"         
                outName = os.path.basename(outFile)

                 # Open log file to get a current list of previously-processed Parks
                '''
                env.workspace = outGdb
                rasterList = arcpy.ListRasters("*", "GRID")
                if outName in rasterList:
                    pass
                else:
                    continue
                '''                               
                logFile = open(logFileName, "r")
                lf_read = logFile.read()
                logFile.close()

                
                testLine = "Successfully Processed - " + outFile   #Checking logfile for this syntax - if there then already processed
                if testLine in lf_read:
                    
                    messageTime = timeFun()
                    logFile = open(logFileName, "a")
                    scriptMsg = "Skipping - " + outFile + "_has been previously processed" + messageTime
                    logFile.write(scriptMsg + "\n")
                    logFile.close()
                    continue
                else:
                    
                    ###########
                    # Clip File
                    ###########
                    if park.lower() == prefix:

                        # Execute Slope
                        outMeasurement = "DEGREE"
                        outSlope = Slope(file, outMeasurement, "")

                        # Save the output 
                        outSlope.save(outFile)
                        #arcpy.Clip_management(file, "#", outFile, clipFile, "", "ClippingGeometry" )
                        
                        #Write log message
                                    
                        logFile = open(logFileName, "a")
                        messageTime = timeFun()            
                        scriptMsg = "Successfully Processed - " + outFile + " - " + messageTime 
                        
                        logFile.write(scriptMsg + "\n")
                        logFile.close()    
                        fileCount = fileCount + 1
                    else:
                        continue
                lineCount = lineCount + 1
            
except:
    messageTime = timeFun()
    scriptMsg = "Exiting Create_Slope.py  Error" 
    print "Exiting Create_Slope.py  Exited\nSee log file " + logFileName + " for more details - " + messageTime
    logFile = open(logFileName, "a")
    logFile.write(scriptMsg + "\n")
    logFile.close()
    
finally:
    messageTime = timeFun()
    scriptMsg = "Finished Create_Slope.py  " + " << " + messageTime
    logFile = open(logFileName, "a")
    logFile.write(scriptMsg + "\n")
    logFile.close()
    