###########################################
# DistanceAirports.py
# Draft version Date: November 13, 2012
# Script Written By:  Kirk Sherrill NPS-NRPC-I&M GIS Program
# 
# Abstract: Using an input table euclidean disances to designated airports feature classes are calculated.  
# Input table must define a Name (column 1), and the file path (column 2) to the respective airport feature class data
# Airport featureclass data was classified by combinations of airport types and airport usage enplanement numbers.
##########################################

###################################################
# Start of Parameters requiring set up.
###################################################

## Required Data Layers
workspace = "Y:\\Projects_2012\\Sounds2012\\Data\\airports\\workspace\\"            ##Scratch workspace

inTable = "Y:\\Projects_2012\\Sounds2012\\Data\\airports\\Airport_List.csv"       ##Table with Airport files to be processed
outGdb = "Y:\\Projects_2012\\Sounds2012\\Data\\airports\\Airport_NTAD_Distance.gdb\\"    ##Out Geodatabase
cellSize = 90           #Output Distance raster cell size
snapRaster = "O:\\Products\\Seamless\\GeoTIFFs\\Landcover_Level1_NLCD2006.tif"            ##Snap Raster
extentRaster = "Y:\\Projects_2012\\Sounds2012\\Continental\\Continental_Boundary.gdb\\Lower_48_boundary_Albers_buf_50mi"  ##Extent Raster
logFileName = workspace + "AA_Distance_Airports.txt"

###################################################
# End of Parameters requiring set up.
###################################################

# Import system modules
import sys, string, os, arcgisscripting, math, arcpy, glob
from arcpy import env
from arcpy.sa import *

# Check out any necessary licenses
arcpy.CheckOutExtension("Spatial")
# Set overwrite files to True 
arcpy.geoprocessing.env.overwriteOutput = True
# Set geoprocessing work space
arcpy.geoprocessing.env.workspace = workspace


def timeFun():          #Function to Grab Time
    from datetime import datetime
    b=datetime.now()
    messageTime = b.isoformat()
    return messageTime

#################################################
##Beginning routine to run indivual Fuzzy Viewsheds per development point
#################################################
try:
    arcpy.env.extent = extentRaster
    arcpy.env.snapRaster = snapRaster    
    env.workspace = outGdb  

    if os.path.exists(logFileName):
        pass
    else:
        logFile = open(logFileName, "w")
        logFile.close()

    readFile = open(inTable, "r")
    lineCount  = 1
    for line in readFile:               #Loop Per File
        if lineCount == 1:
            lineCount = lineCount + 1
            pass
        else:
    
            splitLine = line.split(",")
            name = str(splitLine[0])
            file = str(splitLine[1])
            cellRas = cellSize
            outFile = outGdb + name
            
            # Open log file to get a current list of previously-processed Parks
            lf_read = ""
            logFile = open(logFileName, "r")
            lf_read = logFile.read()
            logFile.close()
            
            testLine = "Successfully Processed - " + file   #Checking logfile for this syntax - if there then already processed
            if testLine in lf_read:
                
                messageTime = timeFun()
                logFile = open(logFileName, "a")
                scriptMsg = "Skipping - " + file + "_has been previously processed" + messageTime
                logFile.write(scriptMsg + "\n")
                logFile.close()
                continue
            else:
                    
                cellRas = cellSize
                outEucDistance = EucDistance(file, "", cellRas, "")
                outEucDistance.save(outFile)
            
           
                #Write log message
                logFile = open(logFileName, "a")
                messageTime = timeFun()            
                scriptMsg = "Successfully Processed - " + file + " " + messageTime 
                
                logFile.write(scriptMsg + "\n")
                logFile.close()    
                
                lineCount = lineCount + 1
            
except:
    messageTime = timeFun()
    scriptMsg = "Exiting DistanceAirports.py Error" 
    print "Exiting DistanceAirports.py Exited\nSee log file " + logFileName + " for more details - " + messageTime
    logFile = open(logFileName, "a")
    logFile.write(scriptMsg + "\n")
    logFile.close()
    
finally:
    messageTime = timeFun()
    scriptMsg = "Finished DistanceAirports.py " + " << " + messageTime
    logFile = open(logFileName, "a")
    logFile.write(scriptMsg + "\n")
    logFile.close()
    