###########################################
# distanceStreamOrder.py 
# Draft version Date: April 18th, 2012
# Script Written By:  Kirk Sherrill NPS-NRPC-I&M GIS Program
# 
# Abstract:  
#   For NHDPlus region (.gdb), using NHDFlowline and SOSC stream order table, a distance
#   to stream by stream order is calculated. Stream order can be defined as either Strahler
#   (SO) or Strahler Calculator (SC).   
##########################################

###################################################
# Start of Parameters requiring set up.
###################################################

## Required Data Layers
workspace = "E:\\Sounds2012\\workspace\\"            ##Scratch workspace
outWorkspace = "E:\\Sounds2012\\NHD_Plus\\"        ##output Workspace
processList = "E:\\Sounds2012\\NHD_Plus\\Table\\NHDPlusList.csv"           ##List of NHDPlus regions to be processed.
streamOrder = 4                         ##Stream order greater than by which distance to stream calculations are performed.
orderType = "StrahlerCal"              ##('StrahlerCal'|'Strahler') Stream Order calculation type (Strahler Calculator or Strahler Order
snapRaster = "D:\\SolarPEIS\\nlcd2006_nc"       ##Snap Raster
memory = "Yes"                      ##('Yes'|'No')      If processing with memory is desired

logFileName = workspace + "DistanceStreamOrder.txt"    #Logfile Name
###################################################
# End of Parameters requiring set up.
###################################################

# Import system modules
import sys, string, os, arcgisscripting, math, arcpy, glob
from arcpy import env
from arcpy.sa import *

# Check out any necessary licenses
arcpy.CheckOutExtension("Spatial")
# Set overwrite files to True 
arcpy.geoprocessing.env.overwriteOutput = True
# Set geoprocessing work space
arcpy.geoprocessing.env.workspace = workspace

if os.path.exists(logFileName):
    pass
else:
    logFile = open(logFileName, "w")
    logFile.close()


def timeFun():          #Function to Grab Time
    from datetime import datetime
    b=datetime.now()
    messageTime = b.isoformat()
    return messageTime

#################################################
##Beginning routine to run indivual Fuzzy Viewsheds per development point
#################################################
try:
        
    arcpy.env.snapRaster = snapRaster
    readFile = open(processList, "r")
    indexFileName = workspace + "DistanceStreamOrder_Index.txt"  ## Index File used for memory - windows scheduler processing is selected.
    lineCount = 1
    for line in readFile:               #Loop Per Park
        if lineCount == 1:
            lineCount = lineCount + 1
            pass
        else:
            
            split = line.split(",")
            region = str(split[0])
            outGdbFull = outWorkspace + region + ".gdb"
            orderTable = outGdbFull + "\\sosc"
            inFile = outGdbFull + "\\nhdflowline_albers" 
            outStrm = outGdbFull + "\\nhdflowline_albers" + "_" + orderType + "_" + str(streamOrder)
            outFile = outGdbFull + "\\Distance_" + region + "_" + orderType + "_" + str(streamOrder)

            lf_read = ""
            logFile = open(logFileName, "r")
            lf_read = logFile.read()
            logFile.close()

            testLine = "Completed " + outFile
            if testLine in lf_read:
                
                messageTime = timeFun()
                logFile = open(logFileName, "a")
                scriptMsg = "Skipping Point - " + outFile + " - has been previously processed" + messageTime
                logFile.write(scriptMsg + "\n")
                logFile.close()
                continue

            else: 

                base = os.path.basename(orderTable)
                base1 = base.split(".")
                newName = str(base1[0])
                gdbTable = outGdbFull + "\\" + newName

                ##Making Feature Layer

                arcpy.MakeFeatureLayer_management(inFile,"strLayer")
                joinField = "COMID"
                #arcpy.AddIndex_management (gdbTable, "COMID", "COMIDIndex", "NON_UNIQUE", "NON_ASCENDING")

                ##Creating join between feature layer and the stream order table
                arcpy.AddJoin_management("strLayer", joinField, gdbTable, joinField, "KEEP_COMMON")

                if orderType == "StrahlerCal":
                    exp = "sosc.SC"
                elif orderType == "Strahler":
                    exp = "sosc.SO"
                else:
                    break
                
                ##Selecting the appropriate streams for the subsequent distance to stream calculation
                syntax = exp + " > " + str(streamOrder)    
                arcpy.Select_analysis("strLayer", outStrm, syntax)  

                arcpy.env.workspace = outGdbFull
                arcpy.env.extent = inFile                    
                cellSize = 30
                outEucDistance = EucDistance(outStrm, "", cellSize, "")
                outEucDistance.save(outFile)

                messageTime = timeFun()
                logFile = open(logFileName, "a")
                
                scriptMsg = "Completed " + outFile + " - " + messageTime
                logFile.write(scriptMsg + "\n")
                logFile.close()
                lineCount = lineCount + 1

                if memory.lower() == "yes":
                    break
                else:
                    pass
                
            
except:
    messageTime = timeFun()
    scriptMsg = "SelectByStreamOrder.py Error" 
    print "SelectByStreamOrder.py Exited\nSee log file " + logFileName + " for more details - " + messageTime
    
    
finally:
    messageTime = timeFun()
    scriptMsg = "Select By Attribute " + " << " + messageTime
   
    