###########################################
# Distance_Water.py
# Draft version Date: 2012/2/2
# Script Written By:  Kirk Sherrill NPS-NRPC-I&M GIS Program
# 
# Abstract:  Calcluates eucilidean distance from NHD streams within the defined area of analysis
##########################################

###################################################
# Start of Parameters requiring set up.
###################################################

## Required Data Layers
workspace = "D:\\Sounds2012\\workspace\\"            ##Scratch workspace

prefix = "Distance"          ##prefix for output file (will be prefix_{park}) and the feature dataset name as well
outWorkspace = "D:\\Sounds2012\\NHD_Waterbody_Distance.gdb\\"    ##Out Geodatabase
sourceFile = "D:\\Sounds2012\\NHD_Waterbody.gdb\\"   ##workspace with source files
sourceFilePrefix = "Waterbody"          ##Prefix for sourceFile 
bufferFile = "D:\\Sounds2012\\AOI_CONUS.gdb\\buf_10mile\\"          ##Workspace with Buffer files
bufferSuffix = "10mile"         ##Suffix for buffer file
parkList = "D:\\Sounds2012\\Sites_CONUS_2012.csv"           ##List of Parks to Process
snapRaster = "O:\\SourceData\\USData\\Landcover\\NLCD_2006\\Landcover_2006\\nlcd2006_landcover_2-14-11.img"            ##Snap Raster

logFileName = workspace + "AA_Distance_Waterbody_NHD.txt"


###################################################
# End of Parameters requiring set up.
###################################################

# Import system modules
import sys, string, os, arcgisscripting, math, arcpy, glob
from arcpy import env
from arcpy.sa import *

# Check out any necessary licenses
arcpy.CheckOutExtension("Spatial")
# Set overwrite files to True 
arcpy.geoprocessing.env.overwriteOutput = True
# Set geoprocessing work space
arcpy.geoprocessing.env.workspace = workspace


def timeFun():          #Function to Grab Time
    from datetime import datetime
    b=datetime.now()
    messageTime = b.isoformat()
    return messageTime

#################################################
##Beginning routine to run indivual Fuzzy Viewsheds per development point
#################################################
try:
    arcpy.env.snapRaster = snapRaster    
    readFile = open(parkList, "r")

    if os.path.exists(logFileName):
        pass
    else:
        logFile = open(logFileName, "w")
        logFile.close()
    
    
    lineCount = 1
    for line in readFile:               #Loop Per Park
        if lineCount == 1:
            lineCount = lineCount + 1
            pass
        else:
            split = line.split(",")
            park = str(split[0])
            siteId = str(split[1])
            #siteId = site + "_" + park
            roads = sourceFile + sourceFilePrefix + "_" + siteId         #Park Specific file to do the clipping/Masking - Not Dynamic
            buffer = bufferFile + "\\" + siteId + "_" + bufferSuffix
            outFC = prefix  #Output feature class    
            
            outFile = outWorkspace + prefix + "_" + siteId           #Output File

             # Open log file to get a current list of previously-processed Parks
            lf_read = ""
            logFile = open(logFileName, "r")
            lf_read = logFile.read()
            logFile.close()
            
            testLine = "Successfully Processed - " + siteId   #Checking logfile for this syntax - if there then already processed
            if testLine in lf_read:
                
                messageTime = timeFun()
                logFile = open(logFileName, "a")
                scriptMsg = "Skipping - " + siteId + "_has been previously processed" + messageTime
                logFile.write(scriptMsg + "\n")
                logFile.close()
                continue
            else:
                
                ###########
                # Distance Per File
                ###########
                
                #arcpy.env.extent = roads
                arcpy.env.extent = buffer
                countRoads = arcpy.GetCount_management(roads)
                countStr = str(countRoads)
                if int(countStr) != 0:
                    
                    cellSize = 30
                    outEucDistance = EucDistance(roads, "", cellSize, "")
                    outEucDistance.save(outFile)
                else:
                    pass
               
                #Write log message
                            
                logFile = open(logFileName, "a")
                messageTime = timeFun()            
                scriptMsg = "Successfully Processed - " + siteId + " " + messageTime 
                
                logFile.write(scriptMsg + "\n")
                logFile.close()    
                
                lineCount = lineCount + 1
            
except:
    messageTime = timeFun()
    scriptMsg = "Exiting Distance_Water.py Error" 
    print "Exiting Distance_Water.py Exited\nSee log file " + logFileName + " for more details - " + messageTime
    logFile = open(logFileName, "a")
    logFile.write(scriptMsg + "\n")
    logFile.close()
    
finally:
    messageTime = timeFun()
    scriptMsg = "Finished Distance_Water.py" + " << " + messageTime
    logFile = open(logFileName, "a")
    logFile.write(scriptMsg + "\n")
    logFile.close()
    