###########################################
# Extent_From_Feature_Class.py - Creates DEM from 1 Deg USGS DEM data - on local
# Draft version Date: May 17, 2011
# Script Written By:  Kirk Sherrill NPS-NRPC-I&M GIS Program
# 
# Abstract:  Per feature in the defined feature class the bounding box extent will be exported to a .csv file. 
# 
##########################################

###################################################
# Start of Parameters requiring set up.
###################################################

## Required Data Layers
workspace = "Y:\\Projects_2012\\Sounds2012\\workspace\\"                                      ## Setting output workspace variable
file = "Y:\\Projects_2012\\Sounds2012\\Data\\SoundSites2012.gdb\\"                          ##Feature Class or workspace with feature class Files 
fileType = "single"                   ##Feature class type - "single" or "multiple" files?

extentList = "Y:\\Projects_2012\\Sounds2012\\workspace\\Sites_CONUS_2012_Test_Extent.csv"      ##Output List of Park Extents per defined bufer distance (i.e. 25, etc.)
logFileName = workspace + "\\AA_Extent_10mileAOI_Sounds.txt"                                       ## Log File

###################################################
# End of Parameters requiring set up.
###################################################

############################################ 
## Below are paths which are hard coded

# Import system modules
import sys, string, os, arcgisscripting, math, arcpy, glob
from arcpy import env
from arcpy.sa import *

# Check out any necessary licenses
arcpy.CheckOutExtension("Spatial")
# Set overwrite files to True 
arcpy.geoprocessing.env.overwriteOutput = True
# Set geoprocessing work space
arcpy.geoprocessing.env.workspace = workspace + "Source\\" 

##################################
# Checking for working directories
##################################

if os.path.exists(workspace):
    pass
else:
    os.makedirs(workspace)


def timeFun():          #Function to Grab Time
    from datetime import datetime
    b=datetime.now()
    messageTime = b.isoformat()
    return messageTime

if os.path.exists(logFileName):
    pass
else:
    logFile = open(logFileName, "w")    #Creating index file if it doesn't exist
    logFile.close()

if os.path.exists(extentList):
    pass
else:
    extentFile = open(extentList, "w")    #Creating index file if it doesn't exist
    header = "siteId_Park,ULLAT,ULLON,LRLAT,LRLON"
    extentFile.write(header + "\n")
    extentFile.close()

#################################################
##Beginning routine to run indivual Fuzzy Viewsheds per development point
#################################################
try:
    # Set geoprocessing work space per park
    
    if fileType.lower() == "single":
        arcpy.geoprocessing.env.workspace = file
        fcList = arcpy.ListFeatureClasses("*CONUS_SiteLocations_GCS_buf_10mile")
        for fc in fcList:
            arcpy.geoprocessing.env.workspace = fc
            sCur = arcpy.SearchCursor(fc)
            name = str(fc)
            for row in sCur:
                
                siteId_Park=str(row.SiteID_Park)
                geom=row.shape
                ext=geom.extent
                yMax = str(ext.YMax)
                yMin = str(ext.YMin)
                xMax = str(ext.XMax)
                xMin = str(ext.XMin)
                outSyntax = siteId_Park + "," + yMax + "," + xMin + "," + yMin + "," + xMax

                extentFile = open(extentList, "a")
                extentFile.write(outSyntax + "\n")
                extentFile.close()

                logFile = open(logFileName, "a")
                scriptMsg = "Processed " + siteId_Park  
                logFile.write(scriptMsg + "\n")
                logFile.close()
    else:

        arcpy.geoprocessing.env.workspace = file
        fcList = arcpy.ListFeatureClasses()

        for fc in fcList:
            fcSplit = fc.split("_")
            name = str(fcSplit[0]) + "_" + str(fcSplit[1])           #change as needed          
            arcpy.geoprocessing.env.workspace = fc
            sCur = arcpy.SearchCursor(fc)
            
            for row in sCur:
                geom=row.shape
                ext=geom.extent
                yMax = str(ext.YMax)
                yMin = str(ext.YMin)
                xMax = str(ext.XMax)
                xMin = str(ext.XMin)
                outSyntax = name + "," + yMax + "," + xMin + "," + yMin + "," + xMax

                extentFile = open(extentList, "a")
                extentFile.write(outSyntax + "\n")
                extentFile.close()

                logFile = open(logFileName, "a")
                scriptMsg = "Processed " + name  
                logFile.write(scriptMsg + "\n")
                logFile.close()
            
            
            
except:
    messageTime = timeFun()
    scriptMsg = "Extent_From_Feature_Class.py Error" 
    print "Exiting Extent_From_Feature_Class.py Exited\nSee log file " + logFileName + " for more details - " + messageTime
    logFile = open(logFileName, "a")
    logFile.write(scriptMsg + "\n")
    logFile.close()
    
finally:
    messageTime = timeFun()
    scriptMsg = "Extent_From_Feature_Class.py complete" + " << " + messageTime
    logFile = open(logFileName, "a")
    logFile.write(scriptMsg + "\n")
    logFile.close()
    