###########################################
# ExtractDistAirports.py
# Draft version Date: Nov 14, 2012
# Script Written By:  Kirk Sherrill NPS-NRPC-I&M GIS Program
# 
# Abstract:  Extracts Point Values from Distance to Airports Rasters
# 
#
##########################################

###################################################
# Start of Parameters requiring set up.
###################################################

## Required Data Layers
workspace = "Y:\\Projects_2012\\Sounds2012\\Data\\airports\\workspace\\"            ##Scratch workspace
pointFile = "Y:\Projects_2012\\Sounds2012\\Data\\airports\\Airport_NTAD_Distance.gdb\\DistanceAirportsMetrics"       ##Point file to extract raster files
rasterWorkspace = "Y:\\Projects_2012\\Sounds2012\\Data\\airports\\Airport_NTAD_Distance.gdb\\"                 ##WorkSpace with Distance to Airport Rasters
outGdb = "Y:\\Projects_2012\\Sounds2012\\Data\\airports\\Airport_NTAD_Distance.gdb\\"           ##Geodatabase to house output extracted feature point files 

logFileName = workspace + "AA_ExtractDistAirports.txt"   ##LogFile Name

###################################################
# End of Parameters requiring set up.
###################################################

# Import system modules
import sys, string, os, arcgisscripting, math, arcpy, glob
from arcpy import env
from arcpy.sa import *

# Check out any necessary licenses
arcpy.CheckOutExtension("Spatial")
# Set overwrite files to True 
arcpy.geoprocessing.env.overwriteOutput = True
# Set geoprocessing work space
arcpy.geoprocessing.env.workspace = workspace



def timeFun():          #Function to Grab Time
    from datetime import datetime
    b=datetime.now()
    messageTime = b.isoformat()
    return messageTime

#################################################
##Beginning routine to run indivual Fuzzy Viewsheds per development point
#################################################
try:
        
    error = ""
    if os.path.exists(logFileName):
        pass
    else:
        logFile = open(logFileName, "w")
        logFile.close()
    
    env.workspace = rasterWorkspace
    rasterList = arcpy.ListRasters("*", "GRID")
    
    for ras in rasterList:               #Loop Per Park
                    
        #split = ras.split(",")
        name = str(ras)         #Name of Raster
        field = str(ras)        #Output Field name        
        fullPath = rasterWorkspace + name
        # Open log file to get a current list of previously-processed Parks
        lf_read = ""
        logFile = open(logFileName, "r")
        lf_read = logFile.read()
        logFile.close()
        
        testLine = "Successfully Processed - " + name  #Checking logfile for this syntax - if there then already processed
        if testLine in lf_read:
            
            messageTime = timeFun()
            logFile = open(logFileName, "a")
            scriptMsg = "Skipping - " + name + "_has been previously processed" + messageTime
            logFile.write(scriptMsg + "\n")
            logFile.close()
            continue
        else:
            
            ###########
            # Extract Point
            ###########
                          
            
            #expression = "[[" + '"' + name + '"' + " ," + '"' + field + '"' + "]]"    
            ExtractMultiValuesToPoints(pointFile, ras, "")
            print "\n" + arcpy.GetMessages() + "\n"
            '''
            expression = "!" + siteId + "_" + rasterSuffix + "!"  #The field which is being calculated to the new field
            arcpy.AddField_management(pointFile, fieldName, "Float")
            arcpy.CalculateField_management(pointFile, fieldName, expression,"PYTHON")
            '''
                        
            logFile = open(logFileName, "a")
            messageTime = timeFun()            

            scriptMsg = "Successfully Processed - " + name + " " + messageTime 
            logFile.write(scriptMsg + "\n")
            logFile.close()    
            
            lineCount = lineCount + 1
            
except:
    error = "Processing Error"
    messageTime = timeFun()
    scriptMsg = "Exiting ExtractDistAirports.py Error" 
    print "Exiting ExtractDistAirports.py Exited\nSee log file " + logFileName + " for more details - " + messageTime
    logFile = open(logFileName, "a")
    logFile.write(scriptMsg + "\n")
    logFile.close()
    
finally:
    if error == "":
        messageTime = timeFun()
        scriptMsg = "Finished ExtractDistAirports.py " + " << " + messageTime
        logFile = open(logFileName, "a")
        logFile.write(scriptMsg + "\n")
        logFile.close()
    else:
        exitSystem = "Error in Processing"
        print exitSystem