###########################################
# Extract_Points.py
# Draft version Date: May 17, 2011
# Script Written By:  Kirk Sherrill NPS-NRPC-I&M GIS Program
# 
# Abstract:  Extracts Point Values from Rasters
# 
#
##########################################

###################################################
# Start of Parameters requiring set up.
###################################################

## Required Data Layers
workspace = "Y:\\Projects_2012\\Sounds2012\\workspace\\"            ##Scratch workspace
pointWorkspace = "Y:\\Projects_2012\\Sounds2012\\Data\\SoundSites2012.gdb\\sites_albers\\"       ##Point Data Files to used to extract raster data
rasterWorkspace = "Y:\\Projects_2012\\Sounds2012\\Data\\NHD_Waterbody_Distance.gdb\\"                 ##WorkSpace with Rasters to be extracted 
rasterSuffix = "Distance"         ##Suffix per raster file
outGdb = "Y:\\Projects_2012\\Sounds2012\\Data\\test\\Extract_Points.gdb\\"           ##Geodatabase to house output table 

siteList = "Y:\\Projects_2012\\Sounds2012\\Tables\\Sites_CONUS_2012_Test.csv"           ##List of Sites to Process
outTable = "Distance_Waterbody_Table"       ##Name of final exported table with point values per point feature class - default export is to outGdb
fieldName = "DistanceWaterBody"               ##Field name of extracted in output table

logFileName = workspace + "AA_Extract_NHD_Waterbody_Distance.txt"   ##LogFile Name


###################################################
# End of Parameters requiring set up.
###################################################

# Import system modules
import sys, string, os, arcgisscripting, math, arcpy, glob
from arcpy import env
from arcpy.sa import *

# Check out any necessary licenses
arcpy.CheckOutExtension("Spatial")
# Set overwrite files to True 
arcpy.geoprocessing.env.overwriteOutput = True
# Set geoprocessing work space
arcpy.geoprocessing.env.workspace = workspace



def timeFun():          #Function to Grab Time
    from datetime import datetime
    b=datetime.now()
    messageTime = b.isoformat()
    return messageTime

#################################################
##Beginning routine to run indivual Fuzzy Viewsheds per development point
#################################################
try:
        
    readFile = open(siteList, "r")

    if os.path.exists(logFileName):
        pass
    else:
        logFile = open(logFileName, "w")
        logFile.close()
    
    
    lineCount = 1
    for line in readFile:               #Loop Per Park
        if lineCount == 1:
            lineCount = lineCount + 1
            pass
        else:
            
            split = line.split(",")
            park = str(split[0])
            siteId = str(split[1])
            #siteId = site + "_" + park
            
            pointFile = pointWorkspace + siteId + "_site"
            #rasterFile = rasterWorkspace + siteId + "_" + rasterSuffix    #Depending upon the naming convention used for the raster file
            rasterFile = rasterWorkspace + rasterSuffix + "_" + siteId
            rasterFileBase = os.path.basename(rasterFile)
            # Open log file to get a current list of previously-processed Parks
            lf_read = ""
            logFile = open(logFileName, "r")
            lf_read = logFile.read()
            logFile.close()
            
            testLine = "Successfully Processed - " + siteId   #Checking logfile for this syntax - if there then already processed
            if testLine in lf_read:
                
                messageTime = timeFun()
                logFile = open(logFileName, "a")
                scriptMsg = "Skipping - " + siteId + "_has been previously processed" + messageTime
                logFile.write(scriptMsg + "\n")
                logFile.close()
                continue
            else:
                
                ###########
                # Extract Point
                ###########
                arcpy.env.workspace = rasterWorkspace
                rasterList = arcpy.ListRasters("*", "GRID")
                rasterBase = os.path.basename(rasterFile)
                extractField = rasterSuffix + "_" + siteId

                #Checking if Field Name exists if Yes delete field
                arcpy.env.workspace = pointFile
                fieldList = arcpy.ListFields(pointFile)

                for field in fieldList:
                    if str(field.name) == extractField: 
                        arcpy.DeleteField_management(pointFile, extractField) 
                    else:
                        pass

                arcpy.env.workspace = outGdb
                if rasterBase in rasterList:
                    
                    #inList = '[["' + rasterFileBase + '", "' + fieldName + '"]]'
                    ExtractMultiValuesToPoints(pointFile, rasterFile, "")
                else:
                    continue
                
                #expression = "!" + siteId + "_" + rasterSuffix + "!"       #Depending upon the naming convention used for the raster file
                expression = "!" + rasterSuffix + "_" + siteId + "!"  #The field which is being calculated to the new field
                arcpy.AddField_management(pointFile, fieldName, "Float")
                arcpy.CalculateField_management(pointFile, fieldName, expression,"PYTHON")
                
                arcpy.env.workspace = outGdb
                tableList = arcpy.ListTables(outTable)
                                

                if len(tableList)< 1:
                    arcpy.TableToTable_conversion(pointFile, outGdb, outTable, "")    

                else:                
                    appendTable = outGdb + "\\" + outTable
                    outTableTemp = siteId + "_All_Temp"
                    tempFull = outGdb + "\\" + outTableTemp
                    arcpy.TableToTable_conversion(pointFile, outGdb, outTableTemp, "")
                    arcpy.Append_management(tempFull, appendTable, "NO_TEST","","")
                    arcpy.Delete_management(outTableTemp)
                
                #Write log message
                            
                logFile = open(logFileName, "a")
                messageTime = timeFun()            
                scriptMsg = "Successfully Processed - " + siteId + " " + messageTime 
                logFile.write(scriptMsg + "\n")
                logFile.close()    
                
                lineCount = lineCount + 1
            
except:
    messageTime = timeFun()
    scriptMsg = "Exiting Extract_Points.py Error" 
    print "Exiting Extract_Points.py Exited\nSee log file " + logFileName + " for more details - " + messageTime
    logFile = open(logFileName, "a")
    logFile.write(scriptMsg + "\n")
    logFile.close()
    
finally:
    messageTime = timeFun()
    scriptMsg = "Finished Extract_Points.py " + " << " + messageTime
    logFile = open(logFileName, "a")
    logFile.write(scriptMsg + "\n")
    logFile.close()
    