###########################################
# ExtractPointsDistStrmOrder.py
# Draft version Date: May 17, 2011
# Script Written By:  Kirk Sherrill NPS-NRPC-I&M GIS Program
# 
# Abstract:  Extracts distance to streams by stream order type raster values from
# the respective NHDPlus region geodatabase which is coincident with the defined
# study site point location.
#
##########################################

###################################################
# Start of Parameters requiring set up.
###################################################

## Required Data Layers
workspace = "I:\\NHD_Plus\\NHD_Plus\\workspace\\"            ##Scratch workspace
rasterWorkspace = "I:\\NHD_Plus\\NHD_Plus\\"      ##Directory with NHDPlus region geodatabases
rasterSuffix = "StrahlerCal_3"              ##Standardized raster file(s) suffix (e.g.StrahlerCal_3), needed for syntax logic.

pointWorkspace = "Y:\\Projects_2012\\Sounds2012\\Data\\SoundSites2012.gdb\\sites_albers\\"    ##Location with study site point feature classes
pointSuffix = "site"   ##Standardized point file suffix (e.g.site), needed for syntax logic.

outGdb = "I:\\NHD_Plus\\NHD_Plus\\DistanceByStreamOrder.gdb"  ##Geodatabase to house the outTable with the distance values per study site
outTable = "Distance_StrahlerCal_3"      ##Name of final exported table with point values per point feature class.
fieldName = "StrahlerCal_3"              ##Field name in outTable with raster point values per study site  (i.e. StrahlerCal_3 or StrahlerCal_1).

siteList = "Y:\\Projects_2012\\Sounds2012\\Tables\\NHDPlusList.csv"   ##List of study sites and the associated NHDPlus region .gdb it is spatially coincident with.

logFileName = workspace + "AA_Extract_Distance_StrahlerCal_3.txt"   ##LogFile name

###################################################
# End of Parameters requiring set up.
###################################################

# Import system modules
import sys, string, os, arcgisscripting, math, arcpy, glob
from arcpy import env
from arcpy.sa import *

# Check out any necessary licenses
arcpy.CheckOutExtension("Spatial")
# Set overwrite files to True 
arcpy.geoprocessing.env.overwriteOutput = True
# Set geoprocessing work space
arcpy.geoprocessing.env.workspace = workspace



def timeFun():          #Function to Grab Time
    from datetime import datetime
    b=datetime.now()
    messageTime = b.isoformat()
    return messageTime

#################################################
##Beginning routine to run indivual Fuzzy Viewsheds per development point
#################################################
try:
        
    readFile = open(siteList, "r")

    if os.path.exists(logFileName):
        pass
    else:
        logFile = open(logFileName, "w")
        logFile.close()
    
    
    lineCount = 1
    for line in readFile:               #Loop Per Park
        if lineCount == 1:
            lineCount = lineCount + 1
            pass
        else:
            
            split = line.split(",")
            park = str(split[0])
            site = str(split[1])
            region = str(split[6])
            siteId = str(split[2]) 
            regionGdb = rasterWorkspace + region + "_" + fieldName + ".gdb"
    
            pointFile = pointWorkspace + siteId + "_" + pointSuffix
            rasterFile = regionGdb + "\\Distance_" + region + "_" + rasterSuffix
            rasterName = os.path.basename(rasterFile)
            # Open log file to get a current list of previously-processed Parks
            lf_read = ""
            logFile = open(logFileName, "r")
            lf_read = logFile.read()
            logFile.close()
            
            testLine = "Successfully Processed - " + siteId   #Checking logfile for this syntax - if there then already processed
            if testLine in lf_read:
                
                messageTime = timeFun()
                logFile = open(logFileName, "a")
                scriptMsg = "Skipping - " + siteId + "_has been previously processed" + messageTime
                logFile.write(scriptMsg + "\n")
                logFile.close()
                continue
            else:
                
                ###########
                # Extract Point
                ###########
                arcpy.env.workspace = outGdb
                
                ExtractMultiValuesToPoints(pointFile, rasterFile, "")
                
                expression = "!" + rasterName + "!"  #Not Dynamic - the field which is being calculated to the new field
                arcpy.AddField_management(pointFile, fieldName, "Float")
                arcpy.CalculateField_management(pointFile, fieldName, expression,"PYTHON")
                
                arcpy.env.workspace = outGdb
                tableList = arcpy.ListTables(outTable)
                                

                if len(tableList)< 1:
                    arcpy.TableToTable_conversion(pointFile, outGdb, outTable, "")    

                else:                
                    appendTable = outGdb + "\\" + outTable
                    outTableTemp = siteId + "_All_Temp"
                    tempFull = outGdb + "\\" + outTableTemp
                    arcpy.TableToTable_conversion(pointFile, outGdb, outTableTemp, "")
                    arcpy.Append_management(tempFull, appendTable, "NO_TEST","","")
                    arcpy.Delete_management(tempFull)
                
                #Write log message
                            
                logFile = open(logFileName, "a")
                messageTime = timeFun()            

                scriptMsg = "Successfully Processed - " + siteId + " " + messageTime 
                logFile.write(scriptMsg + "\n")
                logFile.close()    
                
                lineCount = lineCount + 1
            
except:
    messageTime = timeFun()
    scriptMsg = "ExtractPointsDistStrmOrder.py Error" 
    print "Exiting ExtractPointsDistStrmOrder.py Exited\nSee log file " + logFileName + " for more details - " + messageTime
    logFile = open(logFileName, "a")
    logFile.write(scriptMsg + "\n")
    logFile.close()
    
finally:
    messageTime = timeFun()
    scriptMsg = "Finished ExtractPointsDistStrmOrder.py " + " << " + messageTime
    logFile = open(logFileName, "a")
    logFile.write(scriptMsg + "\n")
    logFile.close()
    