###########################################
# ExtractRoadsLCC.py
# Draft version Date: 2012/2/2
# Script Written By:  Kirk Sherrill NPS-NRPC-I&M GIS Program
# 
# Abstract:  By defined area of analysis extracts clip input data
# Note this script is set to clip input data as defined in the siteId_ParkList.csv file
##########################################

###################################################
# Start of Parameters requiring set up.
###################################################

## Required Data Layers
workspace = "Y:\\Projects_2012\\Sounds2012\\workspace\\"            ##Scratch workspace
aoaFile = "Y:\\Projects_2012\\Sounds2012\\Data\\AOI_CONUS.gdb\\buf_10mile\\"   ##Workspace with the Clip/Extract AOI files
aoaSuffix = "10mile"        ##Suffix for AOI files
outGdb1 = "Y:\\Projects_2012\Sounds2012\\Data\\test\\Road_All.gdb\\"          ##Out Gdb file
prefix = "RoadAll"          ##prefix for output file (will be prefix_{siteId_Park})
siteId_ParkList = "Y:\\Projects_2012\\Sounds2012\\Tables\\LCC_Name_SiteId_test.csv"           ##List of siteId_Parks to processed as well as input data to be clipped

logFileName = workspace + "AA_Extract_Roads_Major_LCC_Logfile.txt"              ##LogFile Name


###################################################
# End of Parameters requiring set up.
###################################################

# Import system modules
import sys, string, os, arcgisscripting, math, arcpy, glob
from arcpy import env
from arcpy.sa import *

# Check out any necessary licenses
arcpy.CheckOutExtension("Spatial")
# Set overwrite files to True 
arcpy.geoprocessing.env.overwriteOutput = True
# Set geoprocessing work space
arcpy.geoprocessing.env.workspace = workspace



def timeFun():          #Function to Grab Time
    from datetime import datetime
    b=datetime.now()
    messageTime = b.isoformat()
    return messageTime

#################################################
##Beginning routine to run indivual Fuzzy Viewsheds per development point
#################################################
try:
        
    readFile = open(siteId_ParkList, "r")

    if os.path.exists(logFileName):
        pass
    else:
        logFile = open(logFileName, "w")
        logFile.close()
    
    lineCount = 1
    for line in readFile:               #Loop Per siteId_Park
        if lineCount == 1:
            lineCount = lineCount + 1
            pass
        else:
            split = line.split(",")
            siteId_Park = str(split[1])            #siteid_siteId_Park comb in this case
            split2 = siteId_Park.split("_")
            park = split2[1]
            clipFile = aoaFile + siteId_Park + "_" + aoaSuffix         #siteId_Park Specific file to do the clipping/Masking
            outFile = outGdb1 + prefix + "_" + siteId_Park

            inputData1 = split[4]                   #Column 4 in the siteId_Park list is the path to the Roads data by LCC                  
            inputData = inputData1[:-1]
            lf_read = ""
            logFile = open(logFileName, "r")            
            lf_read = logFile.read()
            logFile.close()
            testLine = "Successfully Created Feature Class " + siteId_Park + "for " + inputData    #Checking logfile for this syntax - if there then already processed
            if testLine in lf_read:
                
                messageTime = timeFun()
                logFile = open(logFileName, "a")
                scriptMsg = "Skipping Feature Class Creation - " + outFC + "for " + siteId_Park + "_has been previously processed" + messageTime
                logFile.write(scriptMsg + "\n")
                logFile.close()
                pass
            else:            
                     
                arcpy.Clip_analysis(inputData, clipFile, outFile)  # If not already clipped to siteId_Park units, use Clip instead of select
               
                #Write log message    
                logFile = open(logFileName, "a")
                messageTime = timeFun()            
                scriptMsg = "Successfully Created Feature Class " + siteId_Park + "for " + inputData + " " + messageTime 
                
                logFile.write(scriptMsg + "\n")
                logFile.close()    
            
            lineCount = lineCount + 1
            
except:
    messageTime = timeFun()
    scriptMsg = "Exiting ExtractRoadsLCC.py Error" 
    print "Exiting ExtractRoadsLCC.py Exited\nSee log file " + logFileName + " for more details - " + messageTime
    logFile = open(logFileName, "a")
    logFile.write(scriptMsg + "\n")
    logFile.close()
    
finally:
    messageTime = timeFun()
    scriptMsg = "Finished ExtractRoadsLCC.py " + " << " + messageTime
    logFile = open(logFileName, "a")
    logFile.write(scriptMsg + "\n")
    logFile.close()
    