###########################################
# ExtractToAOANHDCoastReproject.py
# Draft version Date:   January 2012 
# Script Written By:  Kirk Sherrill NPS-NRPC-I&M GIS Program
# 
# Abstract:  By defined study site extracts out NHD Coast data and reproject to the
# desired projection (Albers)
#
##########################################

###################################################
# Start of Parameters requiring set up.
###################################################

## Required Data Layers
workspace = "E:\\Sounds2012\\workspace\\"            ##Scratch workspace
inputWorkSpace = "E:\\Sounds2012\\NHD\\"       ##Workspace with NHD files to be processed
aoaFile = "E:\\Sounds2012\\AOI_CONUS.gdb\\buf_10mile\\"   ##Workspace with the Clip/Extract AOI files
aoaSuffix = "10mile"        ##Suffix for AOI files
outGdb1 = "E:\\Sounds2012\\NHD_Coast.gdb\\"          ##Out Gdb file
prefix = "Coast"          ##prefix for output file (will be prefix_{siteId_Park})
rpjFile = "E:\\Sounds2012\\SoundSites2012.gdb\\CONUS_SiteLocations_Albers"   ##Reprojection file (i.e. this projection will be used in the reprojection) 
siteId_ParkList = "E:\\Sounds2012\\Sites_CONUS_2012.csv"           ##List of siteId_Parks to Process


logFileName = workspace + "AA_NHD_Coast_10mile_Logfile.txt" ##LogFile Name

###################################################
# End of Parameters requiring set up.
###################################################

# Import system modules
import sys, string, os, arcgisscripting, math, arcpy, glob
from arcpy import env
from arcpy.sa import *

# Check out any necessary licenses
arcpy.CheckOutExtension("Spatial")
# Set overwrite files to True 
#arcpy.geoprocessing.env.overwriteOutput = True
# Set geoprocessing work space
arcpy.geoprocessing.env.workspace = workspace



def timeFun():          #Function to Grab Time
    from datetime import datetime
    b=datetime.now()
    messageTime = b.isoformat()
    return messageTime

#################################################
##Beginning routine to run indivual Fuzzy Viewsheds per development point
#################################################
try:
        
    readFile = open(siteId_ParkList, "r")

    if os.path.exists(logFileName):
        pass
    else:
        logFile = open(logFileName, "w")
        logFile.close()
    
    
    lineCount = 1
    for line in readFile:               #Loop Per siteId_Park
        if lineCount == 1:
            lineCount = lineCount + 1
            pass
        else:
            split = line.split(",")
            siteId_Park = str(split[1])            #siteid_siteId_Park comb in this case
            split2 = siteId_Park.split("_")
            park = split2[1]
            clipFile = aoaFile + siteId_Park + "_" + aoaSuffix         #siteId_Park Specific file to do the clipping/Masking
            outFile = outGdb1 + prefix + "_" + siteId_Park

            inputData =  inputWorkSpace + park + "_HIS.mdb\\Hydrography\\NHD_Coast"           
            inputDataAlbers = inputWorkSpace + park + "_HIS.mdb\\NHD_Coast_Albers"
            base = os.path.basename(inputDataAlbers)
            arcpy.env.workspace = inputWorkSpace + park + "_HIS.mdb\\"
            fcList = arcpy.ListFeatureClasses()
            if base in fcList:
                pass
            else:
                
                arcpy.Project_management(inputData, inputDataAlbers, rpjFile)           #Reprojecting to Albers

            lf_read = ""
            logFile = open(logFileName, "r")            
            lf_read = logFile.read()
            logFile.close()
            testLine = "Successfully Created Feature Class " + siteId_Park + " for " + inputDataAlbers   #Checking logfile for this syntax - if there then already processed
            if testLine in lf_read:
                
                messageTime = timeFun()
                logFile = open(logFileName, "a")
                scriptMsg = "Skipping Feature Class Creation - " + outFile + "for " + siteId_Park + "_has been previously processed" + messageTime
                logFile.write(scriptMsg + "\n")
                logFile.close()
                pass
            else:            

                arcpy.env.ZResolution = "1 Meters"      #Setting Z Resolution
                arcpy.env.MResolution = 1               #Setting M Resolution                     
                arcpy.env.outputZFlag = "Disabled"
                arcpy.env.outputMFlag = "Disabled"

                
                arcpy.Clip_analysis(inputDataAlbers, clipFile, outFile)  # If not already clipped to siteId_Park units, use Clip instead of select
               
                #Write log message    
                logFile = open(logFileName, "a")
                messageTime = timeFun()            
                scriptMsg = "Successfully Created Feature Class " + siteId_Park + " for " + inputDataAlbers + " " + messageTime 
                
                logFile.write(scriptMsg + "\n")
                logFile.close()    
            
            lineCount = lineCount + 1
            
except:
    messageTime = timeFun()
    scriptMsg = "Exiting ExtractToAOANHDCoastReproject.py Error" 
    print "Exiting ExtractToAOANHDCoastReproject.py Exited\nSee log file " + logFileName + " for more details - " + messageTime
    logFile = open(logFileName, "a")
    logFile.write(scriptMsg + "\n")
    logFile.close()
    
finally:
    messageTime = timeFun()
    scriptMsg = "Finished ExtractToAOANHDCoastReproject.py " + " << " + messageTime
    logFile = open(logFileName, "a")
    logFile.write(scriptMsg + "\n")
    logFile.close()
    