###########################################
# ExtractToAOAPoly.py 
# Draft version Date: 
# Script Written By:  Kirk Sherrill NPS-NRPC-I&M GIS Program
# 
# Abstract:  By defined siteId_Park AOI extracts the feature class
##########################################

###################################################
# Start of Parameters requiring set up.
###################################################

## Required Data Layers
workspace = "E:\\Sounds2012\\workspace\\"            ##Scratch workspace
inputData = "E:\\Sounds2012\\NHD\\"       ##Data File/ or Workspace to be processed
aoaFile = "E:\\Sounds2012\\AOI_CONUS.gdb\\buf_10mile_GCS\\"   ##Workspace with the Clip/Extract AOI files
aoaSuffix = "10mileGCS"        ##Suffix for AOI files
outGdb1 = "E:\\Sounds2012\\NHD_Streams.gdb\\"          ##Out Gdb file
prefix = "Streams"          ##Prefix for output file (will be prefix_{siteId_Park})

siteId_ParkList = "E:\\Sounds2012\\Sites_CONUS_2012.csv"           ##List of siteId_Parks to Process

logFileName = workspace + "AA_NHD_Streams_10mile_Logfile.txt"       ##LogFile Name


###################################################
# End of Parameters requiring set up.
###################################################

# Import system modules
import sys, string, os, arcgisscripting, math, arcpy, glob
from arcpy import env
from arcpy.sa import *

# Check out any necessary licenses
arcpy.CheckOutExtension("Spatial")
# Set overwrite files to True 
arcpy.geoprocessing.env.overwriteOutput = True
# Set geoprocessing work space
arcpy.geoprocessing.env.workspace = workspace



def timeFun():          #Function to Grab Time
    from datetime import datetime
    b=datetime.now()
    messageTime = b.isoformat()
    return messageTime

#################################################
##Beginning routine to run indivual Fuzzy Viewsheds per development point
#################################################
try:
        
    readFile = open(siteId_ParkList, "r")

    if os.path.exists(logFileName):
        pass
    else:
        logFile = open(logFileName, "w")
        logFile.close()
    
    
    lineCount = 1
    for line in readFile:               #Loop Per siteId_Park
        if lineCount == 1:
            lineCount = lineCount + 1
            pass
        else:
            split = line.split(",")
            siteId_Park = str(split[1])            #siteid_siteId_Park comb in this case
            clipFile = aoaFile + siteId_Park + "_" + aoaSuffix         #siteId_Park Specific file to do the clipping/Masking
            outFile = outGdb1 + prefix + "_" + siteId_Park
            
            lf_read = ""
            logFile = open(logFileName, "r")            
            lf_read = logFile.read()
            logFile.close()
            testLine = "Successfully Created Feature Class " + siteId_Park + "for " + inputData    #Checking logfile for this syntax - if there then already processed
            if testLine in lf_read:
                
                messageTime = timeFun()
                logFile = open(logFileName, "a")
                scriptMsg = "Skipping Feature Class Creation - " + outFC + "for " + siteId_Park + "_has been previously processed" + messageTime
                logFile.write(scriptMsg + "\n")
                logFile.close()
                pass
            else:            

                arcpy.env.extent = clipFile                     
                arcpy.Clip_analysis(inputData, clipFile, outFile)  # If not already clipped to siteId_Park units, use Clip instead of select
               
                #Write log message    
                logFile = open(logFileName, "a")
                messageTime = timeFun()            
                scriptMsg = "Successfully Created Feature Class " + siteId_Park + "for " + inputData + " " + messageTime 
                
                logFile.write(scriptMsg + "\n")
                logFile.close()    
            
            lineCount = lineCount + 1
            
except:
    messageTime = timeFun()
    scriptMsg = "Exiting ExtractToAOAPoly.py Error" 
    print "Exiting ExtractToAOAPoly.py Exited\nSee log file " + logFileName + " for more details - " + messageTime
    logFile = open(logFileName, "a")
    logFile.write(scriptMsg + "\n")
    logFile.close()
    
finally:
    messageTime = timeFun()
    scriptMsg = "Finished ExtractToAOAPoly.py " + " << " + messageTime
    logFile = open(logFileName, "a")
    logFile.write(scriptMsg + "\n")
    logFile.close()
    