###########################################
# ExtractToAOARaster.py
#
# Draft version Date: June, 2011
# Script Written By:  Kirk Sherrill NPS-NRPC-I&M GIS Program
# 
# Abstract:  Script will loop through a list of study sites and extract the input raster layer to the  
# respective Area of Interest (AOI) (e.g. 1/10/25 mile buffers) feature class for that study site as
# defined in the study site list. 
##########################################

###################################################
# Start of Parameters requiring set up.
###################################################

## Required Data Layers
workspace = "Y:\\Projects_2012\\Sounds2012\\workspace\\"            ##root workspace
inputWorkspace = "O:\\Products\\Seamless\\Road_Density_AllRoads_ESRI2005.gdb\\rdd_all"  ##Input data and or workspace to be clipped                                    
aoaFile = "Y:\\Projects_2012\\Sounds2012\\Data\AOI_CONUS.gdb\\buf_1mile\\"     ##Location with AOA files (1/10/25 mile radius)
aoaSuffix = "1mile"            ##Suffix for AOI file
outGdb = "Y:\\Projects_2012\\Sounds2012\\Data\\test\\RoadDensityAll1mile.gdb\\"         ##output GDB workspace
prefix = "RoadAll"    ##Prefix name for clipped out put
snapRaster = "O:\\Products\\Seamless\\GeoTIFFs\\Landcover_Level2_NLCD2006.tif"   ##raster to snap out rasters to

parkList = "Y:\\Projects_2012\\Sounds2012\\Tables\\Sites_CONUS_2012_Test.csv"           ##List of Parks/Sites to Process.

logFileName = workspace + "AA_LandCover_Test.txt"  ##Log FileName


###################################################
# End of Parameters requiring set up.
###################################################

# Import system modules
import sys, string, os, arcgisscripting, math, arcpy, glob
from arcpy import env
from arcpy.sa import *

# Check out any necessary licenses
arcpy.CheckOutExtension("Spatial")
# Set overwrite files to True 
arcpy.geoprocessing.env.overwriteOutput = True
# Set geoprocessing work space
arcpy.geoprocessing.env.workspace = workspace



def timeFun():          #Function to Grab Time
    from datetime import datetime
    b=datetime.now()
    messageTime = b.isoformat()
    return messageTime

#################################################
##Beginning routine to run indivual Fuzzy Viewsheds per development point
#################################################
try:
        
    arcpy.env.snapRaster = snapRaster
    readFile = open(parkList, "r")
    lineCount = 1
    for line in readFile:               #Loop Per Park
        if lineCount == 1:
            lineCount = lineCount + 1
            pass
        else:
            split = line.split(",")
            park = str(split[0])
            siteIdPark = str(split[1])
            clipFile = aoaFile + siteIdPark + "_" + aoaSuffix         #Park Specific file to do the clipping/Masking

                    
            #dataFiles = glob.glob(inputWorkspace + "*")      
            env.workspace = inputWorkspace
            #dataFiles = arcpy.ListRasters("*", "GRID") #Grab the list of folders containing the files needing to be extracted
            dataFiles = []
            
            dataFiles.append(inputWorkspace)
            fileCount = 1
            for file in dataFiles:          #Data Source Files to be clipped/extracted
                
                            
                filePath = file.split("\\")
                t = len(filePath)
                name = filePath[t-1]
                nameSplit = name.split("_")
                
                outFile = outGdb + prefix + "_" + siteIdPark
                # Open log file to get a current list of previously-processed Parks

                if os.path.exists(logFileName):
                    pass
                else:
                    logFile = open(logFileName, "w")
                    logFile.close()

                lf_read = ""
                logFile = open(logFileName, "r")
                lf_read = logFile.read()
                logFile.close()
                
                testLine = "Successfully Processed - " + siteIdPark   #Checking logfile for this syntax - if there then already processed
                if testLine in lf_read:
                    
                    messageTime = timeFun()
                    logFile = open(logFileName, "a")
                    scriptMsg = "Skipping - " + park + " - for file " + name + "_has been previously processed" + messageTime
                    logFile.write(scriptMsg + "\n")
                    logFile.close()
                    continue
                else:
                    
                    ###########
                    # Clip File
                    ###########
                     
                    arcpy.Clip_management(file, "#", outFile, clipFile, "#", "ClippingGeometry" )
                    
                    #Write log message
                    logFile = open(logFileName, "a")
                    messageTime = timeFun()            
                    scriptMsg = "Successfully Processed - " + siteIdPark + " " + messageTime 
                    
                    logFile.write(scriptMsg + "\n")
                    logFile.close()    
                    fileCount = fileCount + 1

                lineCount = lineCount + 1
            
except:
    messageTime = timeFun()
    scriptMsg = "Exiting ExtractToAOARaster.py Error" 
    print "Exiting ExtractToAOARaster.py Exited\nSee log file " + logFileName + " for more details - " + messageTime
    logFile = open(logFileName, "a")
    logFile.write(scriptMsg + "\n")
    logFile.close()
    
finally:
    messageTime = timeFun()
    scriptMsg = "Finished ExtractToAOARaster.py << " + messageTime
    logFile = open(logFileName, "a")
    logFile.write(scriptMsg + "\n")
    logFile.close()
    