###########################################
# NHDSelectStreamCoast.py
# Draft version Date: Nov 16th, 2011
# Script Written By:  Kirk Sherrill NPS-NRPC-I&M GIS Program
# 
# Abstract:  Extracts National Hydrographic Data (NHD) Stream (FType = 460) and NHD Coast (FType = 566), creating NHD_Stream and NHD_Coast feature classes.
# NHD data by NPS unit was obtained from Ling et al. Water Resources Division, NRSS, NPS. 
##########################################

###################################################
# Start of Parameters requiring set up.
###################################################

## Required Data Layers
workspace = "E:\\Sounds2012\\NHD\\"                                      ## Workspace with NHD mdb files
logFileName = workspace + "NHD_Select_Stream_Coast_2012.txt"            ## Log File


###################################################
# End of Parameters requiring set up.
###################################################

############################################ 
## Below are paths which are hard coded

# Import system modules
import sys, string, os, arcgisscripting, math, arcpy, glob
from arcpy import env
from arcpy.sa import *

# Check out any necessary licenses
arcpy.CheckOutExtension("Spatial")
# Set overwrite files to True 
arcpy.geoprocessing.env.overwriteOutput = True
# Set geoprocessing work space
arcpy.geoprocessing.env.workspace = workspace + "Source\\" 

##################################
# Checking for working directories
##################################

if os.path.exists(workspace):
    pass
else:
    os.makedirs(workspace)


def timeFun():          #Function to Grab Time
    from datetime import datetime
    b=datetime.now()
    messageTime = b.isoformat()
    return messageTime

if os.path.exists(logFileName):
    pass
else:
    logFile = open(logFileName, "w")    #Creating index file if it doesn't exist
    logFile.close()


#################################################
##Beginning routine to run indivual Fuzzy Viewsheds per development point
#################################################
try:
    # Set geoprocessing work space per park
    arcpy.geoprocessing.env.workspace = workspace
    fcList = arcpy.ListWorkspaces("*.mdb")
    for fc in fcList:
        fcStr = str(fc)
        name = os.path.basename(fcStr)
        nameSplit = name.split("_")
        park = str(nameSplit[0])
        
        
        # Open log file to get a current list of previously-processed Parks
        lf_read = ""
        logFile = open(logFileName, "r")
        lf_read = logFile.read()
        logFile.close()
        
        testLine = "Successfully Processed - " + name   #Checking logfile for this syntax - if there then already processed
        if testLine in lf_read:
            
            messageTime = timeFun()
            logFile = open(logFileName, "a")
            scriptMsg = "Skipping - " + name + "_has been previously processed" + messageTime
            logFile.write(scriptMsg + "\n")
            logFile.close()
            continue
        else:

            arcpy.geoprocessing.env.workspace = fc
            gdbName = str(fc)
            inputData = gdbName + "\\Hydrography\\NHDFlowline\\" 
            outStream = gdbName + "\\Hydrography\\NHD_Stream"
            outCoast = gdbName + "\\Hydrography\\NHD_Coast"
            syntax1 = "[FType] = 460"               #note this is hard coded for UNIT_CODE
            syntax2 = "[FType] = 566"
            arcpy.Select_analysis(inputData, outStream, syntax1)           
            arcpy.Select_analysis(inputData, outCoast, syntax2)
            

            logFile = open(logFileName, "a")
            scriptMsg = "Successfully Processed - " + name  
            logFile.write(scriptMsg + "\n")
            logFile.close()

                
                
except:
    messageTime = timeFun()
    scriptMsg = "NHDSelectStreamCoast.py Error" 
    print "Exiting NHDSelectStreamCoast.py Exited\nSee log file " + logFileName + " for more details - " + messageTime
    logFile = open(logFileName, "a")
    logFile.write(scriptMsg + "\n")
    logFile.close()
    
finally:
    messageTime = timeFun()
    scriptMsg = "NHDSelectStreamCoast.py complete" + " << " + messageTime
    logFile = open(logFileName, "a")
    logFile.write(scriptMsg + "\n")
    logFile.close()
    