###########################################
# Proportional.py
# Draft version Date: Nov 16th, 2011
# Script Written By:  Kirk Sherrill NPS-NRPC-I&M GIS Program
# 
# Abstract:  Per NLCD Class level 1 (2006) returns the proportion within the defined AOA in a .csv file.
# For the water and forest level 1 classes - level two proportions are returns.  For Water: Class 11 - Water, Class 12 Snow,
# for Forest: Class 41 - Deciduous Forest, Class 42 - Evergreen Forest, Class 43 - Mixed Forest. 
# http://www.mrlc.gov/nlcd06_leg.php
##########################################

###################################################
# Start of Parameters requiring set up.
###################################################

## Required Data Layers
workspace = "Y:\\Projects_2012\\Sounds2012\\workspace\\"                                      ## Setting output workspace variable
file = "Y:\\Projects_2012\\Sounds2012\\Data\\test\\NLCD1mile.gdb\\"                          ##Feature Class or workspace with NLCD files
outTable = "Y:\\Projects_2012\\Sounds2012\\workspace\\NLCD_proportional_Water_Snow_Level_II_1mile.csv"      ##Out put .csv file
logFileName = workspace + "Proportional_NLCD_Water_Snow_Level_II_1mile.txt"        ## Log File

###################################################
# End of Parameters requiring set up.
###################################################

############################################ 
## Below are paths which are hard coded

# Import system modules
import sys, string, os, arcgisscripting, math, arcpy, glob
from arcpy import env
from arcpy.sa import *

# Check out any necessary licenses
arcpy.CheckOutExtension("Spatial")
# Set overwrite files to True 
arcpy.geoprocessing.env.overwriteOutput = True
# Set geoprocessing work space
arcpy.geoprocessing.env.workspace = workspace + "Source\\" 

##################################
# Checking for working directories
##################################

if os.path.exists(workspace):
    pass
else:
    os.makedirs(workspace)


def timeFun():          #Function to Grab Time
    from datetime import datetime
    b=datetime.now()
    messageTime = b.isoformat()
    return messageTime

if os.path.exists(logFileName):
    pass
else:
    logFile = open(logFileName, "w")    #Creating index file if it doesn't exist
    logFile.close()

if os.path.exists(outTable):
    pass
else:
    extentFile = open(outTable, "w")    #Creating index file if it doesn't exist
    header = "SITEIDPARK,Water,Snow,Developed,Barren,Deciduous,Evergreen,Mixed,Shrubland,Herbaceous,Cultivated,Wetlands"
    extentFile.write(header + "\n")
    extentFile.close()

#################################################
##
try:
    # Set geoprocessing work space per park
    blowedUp = ""
    arcpy.geoprocessing.env.workspace = file
    fcList = arcpy.ListRasters()
    for fc in fcList:
        name = str(fc)

        nameSplit = name.split("_")
        site = str(nameSplit[1])
        park = str(nameSplit[2])
        siteIdPark = site + "_" + park
        
        # Open log file to get a current list of previously-processed Parks
        lf_read = ""
        logFile = open(logFileName, "r")
        lf_read = logFile.read()
        logFile.close()
        
        testLine = "Successfully Processed - " + siteIdPark   #Checking logfile for this syntax - if there then already processed
        if testLine in lf_read:
            
            messageTime = timeFun()
            logFile = open(logFileName, "a")
            scriptMsg = "Skipping - " + siteIdPark + "_has been previously processed" + messageTime
            logFile.write(scriptMsg + "\n")
            logFile.close()
            continue
        else:

            arcpy.geoprocessing.env.workspace = fc
            sCur = arcpy.SearchCursor(fc)
           
            sum = 0
            waterCount = 0
            snowCount = 0
            developedCount = 0
            barrenCount = 0
            deciduousCount = 0
            evergreenCount = 0
            mixedCount = 0
            shrublandCount = 0
            herbCount = 0
            cultivatedCount = 0
            wetlandsCount = 0
            
            for row in sCur:
               
                value = row.Value
                count = row.Count
                sum = sum + count
                if value == 11:
                    waterCount = waterCount + count
                elif value == 12:
                    snowCount = snowCount + count
                elif value == 21 or value == 22 or value == 23 or value == 24:
                    developedCount = developedCount + count
                elif value == 31:
                    barrenCount = barrenCount + count
                elif value == 41:
                    deciduousCount = deciduousCount + count
                elif value == 42: 
                    evergreenCount = evergreenCount + count
                elif value == 43:
                    mixedCount = mixedCount + count
                elif value == 51 or value == 52:
                    shrublandCount = shrublandCount + count
                elif value == 71 or value == 72 or value == 73 or value == 74:
                    herbCount = herbCount + count
                elif value == 81 or value == 82:
                    cultivatedCount = cultivatedCount + count
                elif value == 90 or value == 95:
                    wetlandsCount = wetlandsCount + count
                else:
                    continue
                    
            water = str((waterCount/sum))           #proportion water    
            snow = str((snowCount/sum))
            developed = str((developedCount/sum))         
            barren = str((barrenCount/sum))
            deciduous = str((deciduousCount/sum))
            evergreen = str((evergreenCount/sum))
            mixed = str((mixedCount/sum))
            shrubland = str((shrublandCount/sum))
            herbaceous = str((herbCount/sum))
            cultivated = str((cultivatedCount/sum))
            wetlands = str((wetlandsCount/sum))

            outSyntax = siteIdPark + "," + water + "," + snow + "," + developed + "," + barren + "," + deciduous + "," + evergreen + "," + mixed + "," + shrubland + "," + herbaceous + "," + cultivated + "," + wetlands

            outFile = open(outTable, "a")
            outFile.write(outSyntax + "\n")
            outFile.close()

            logFile = open(logFileName, "a")
            scriptMsg = "Successfully Processed - " + siteIdPark  
            logFile.write(scriptMsg + "\n")
            logFile.close()

            
except:
    blowedUp = "Yes"
    messageTime = timeFun()
    scriptMsg = "Proportional_Water_Forest_Level_II.py Error" 
    print "Proportional_Water_Forest_Level_II.py Exited\nSee log file " + logFileName + " for more details - " + messageTime
    logFile = open(logFileName, "a")
    logFile.write(scriptMsg + "\n")
    logFile.close()
    
finally:

    if blowedUp == "Yes":
        pass
    else:
        messageTime = timeFun()
        scriptMsg = "Proportional_Water_Forest_Level_II.py complete" + " << " + messageTime
        logFile = open(logFileName, "a")
        logFile.write(scriptMsg + "\n")
        logFile.close()
        