###########################################
# Proportional.py
# Draft version Date: Nov 16th, 2011
# Script Written By:  Kirk Sherrill NPS-NRPC-I&M GIS Program
# 
# Abstract:  Per NLCD Class level 1 (2006) returns the proportion within the defined AOA in a .csv file.
# For the water and forest level 1 classes - level two proportions are returns.  For Water: Class 11 - Water, Class 12 Snow,
# for Forest: Class 41 - Deciduous Forest, Class 42 - Evergreen Forest, Class 43 - Mixed Forest. 
# http://www.mrlc.gov/nlcd06_leg.php
##########################################

###################################################
# Start of Parameters requiring set up.
###################################################

## Required Data Layers
workspace = "E:\\Sounds2013\\workspace"                                      ## Setting output workspace variable
file = "E:\\Sounds2013\\LandUse5km.gdb\\"                          ##Feature Class or workspace with NLCD files
outTable = "E:\\Sounds2013\\LandUse_proportional_5km.csv"      ##Out put .csv file
logFileName = workspace + "\\LandUse_proportional_5km.txt"        ## Log File

###################################################
# End of Parameters requiring set up.
###################################################

############################################ 
## Below are paths which are hard coded

# Import system modules
import sys, string, os, arcgisscripting, math, arcpy, glob
from arcpy import env
from arcpy.sa import *

# Check out any necessary licenses
arcpy.CheckOutExtension("Spatial")
# Set overwrite files to True 
arcpy.geoprocessing.env.overwriteOutput = True
# Set geoprocessing work space
arcpy.geoprocessing.env.workspace = workspace + "Source\\" 

##################################
# Checking for working directories
##################################

if os.path.exists(workspace):
    pass
else:
    os.makedirs(workspace)


def timeFun():          #Function to Grab Time
    from datetime import datetime
    b=datetime.now()
    messageTime = b.isoformat()
    return messageTime

if os.path.exists(logFileName):
    pass
else:
    logFile = open(logFileName, "w")    #Creating index file if it doesn't exist
    logFile.close()

if os.path.exists(outTable):
    pass
else:
    extentFile = open(outTable, "w")    #Creating index file if it doesn't exist
    header = "SITEIDPARK,WaterNat,WaterHum,Wetlands,RecCon,Timber,Grazing,Pasture,Cropland,Mining,Park,ExurbanHigh,ExurbanLow,Suburban,UrbanLow,UrbanHigh,Commercial,Industrial,Institutional,Transportation"
    extentFile.write(header + "\n")
    extentFile.close()

#################################################
##
try:
    # Set geoprocessing work space per park
    blowedUp = ""
    arcpy.geoprocessing.env.workspace = file
    fcList = arcpy.ListRasters()
    for fc in fcList:
        name = str(fc)

        nameSplit = name.split("_")
        site = str(nameSplit[1])
        park = str(nameSplit[2])
        siteIdPark = site + "_" + park
        
        # Open log file to get a current list of previously-processed Parks
        lf_read = ""
        logFile = open(logFileName, "r")
        lf_read = logFile.read()
        logFile.close()
        
        testLine = "Successfully Processed - " + siteIdPark   #Checking logfile for this syntax - if there then already processed
        if testLine in lf_read:
            
            messageTime = timeFun()
            logFile = open(logFileName, "a")
            scriptMsg = "Skipping - " + siteIdPark + "_has been previously processed" + messageTime
            logFile.write(scriptMsg + "\n")
            logFile.close()
            continue
        else:

            arcpy.geoprocessing.env.workspace = fc
            sCur = arcpy.SearchCursor(fc)
           
            sum = 0
            waterNatCount = 0
            waterHumCount = 0
            wetlandsCount = 0
            recCount = 0
            timberCount = 0
            grazingCount = 0
            pastureCount = 0
            croplandCount = 0
            miningCount = 0
            parkCount = 0
            exurbanHighCount = 0
            exurbanLowCount = 0
            suburbanCount = 0
            urbanCount = 0
            urbanHighCount = 0
            commercialCount = 0
            industrialCount = 0
            institutionalCount = 0
            transportationCount = 0
            
            for row in sCur:
               
                value = row.Value
                count = row.Count
                sum = sum + count
                if value == 0:
                    waterNatCount = waterNatCount + count
                elif value == 1:
                    waterHumCount = waterHumCount + count
                elif value == 2:
                    wetlandsCount = wetlandsCount + count
                elif value == 3:
                    recCount = recCount + count
                elif value == 4:
                    timberCount = timberCount + count
                elif value == 5: 
                    grazingCount = grazingCount + count
                elif value == 6:
                    pastureCount = pastureCount + count
                elif value == 7:
                    croplandCount = croplandCount + count
                elif value == 8:
                    miningCount = miningCount + count
                elif value == 9:
                    parkCount = parkCount + count
                elif value == 10:
                    exurbanHighCount = exurbanHighCount + count
                elif value == 11:
                    exurbanLowCount = exurbanLowCount + count
                elif value == 12:
                    suburbanCount = suburbanCount + count
                elif value == 13:
                    urbanCount = urbanCount + count
                elif value == 14:
                    urbanHighCount = urbanHighCount + count
                elif value == 15:
                    commercialCount = commercialCount + count
                elif value == 16:
                    industrialCount = industrialCount + count
                elif value == 17:
                    institutionalCount = institutionalCount + count
                elif value == 18:
                    transportationCount = transportationCount + count

                else:
                    continue
                    
            waterNat = str((waterNatCount/sum))              
            waterHum = str((waterHumCount/sum))
            wetlands = str((wetlandsCount/sum))         
            rec = str((recCount/sum))
            timber = str((timberCount/sum))
            grazing = str((grazingCount/sum))
            pasture = str((pastureCount/sum))
            cropland = str((croplandCount/sum))
            mining = str((miningCount/sum))
            park = str((parkCount/sum))
            exurbanHigh = str((exurbanHighCount/sum))
            exurbanLow = str((exurbanLowCount/sum))
            suburban = str((suburbanCount/sum))
            urbanLow = str((urbanCount/sum))
            urbanHigh = str((urbanHighCount/sum))
            commercial = str((commercialCount/sum))
            industrial = str((industrialCount/sum))
            institutional = str((institutionalCount/sum))
            transportation = str((transportationCount/sum))

            outSyntax = siteIdPark + "," + waterNat + "," + waterHum + "," + wetlands + "," + rec + "," + timber + "," + grazing + "," + pasture + "," + cropland + "," + mining + "," + park + "," + exurbanHigh + "," + exurbanLow + "," + suburban + "," + urbanLow + "," + urbanHigh + "," + commercial + "," + industrial + "," + institutional + "," + transportation + ","

            outFile = open(outTable, "a")
            outFile.write(outSyntax + "\n")
            outFile.close()

            logFile = open(logFileName, "a")
            scriptMsg = "Successfully Processed - " + siteIdPark  
            logFile.write(scriptMsg + "\n")
            logFile.close()

            
except:
    blowedUp = "Yes"
    messageTime = timeFun()
    scriptMsg = "Proportional_Water_Forest_Level_II.py Error" 
    print "Proportional_Water_Forest_Level_II.py Exited\nSee log file " + logFileName + " for more details - " + messageTime
    logFile = open(logFileName, "a")
    logFile.write(scriptMsg + "\n")
    logFile.close()
    
finally:

    if blowedUp == "Yes":
        pass
    else:
        messageTime = timeFun()
        scriptMsg = "Proportional_Water_Forest_Level_II.py complete" + " << " + messageTime
        logFile = open(logFileName, "a")
        logFile.write(scriptMsg + "\n")
        logFile.close()
        