###########################################
# AddStreamOrder.py 
# Draft version Date: April 18th, 2012
# Script Written By:  Kirk Sherrill NPS-NRPC-I&M GIS Program
# 
# Abstract:  
# Reprojects nhdflowline data from GCS NAD83 to Albers Equal Area
# Conic USGS version (or other desired projection).  
#
##########################################

###################################################
# Start of Parameters requiring set up.
###################################################

## Required Data Layers
workspace = "E:\\Sounds2012\\workspace\\"            ##Scratch workspace.
outWorkspace = "E:\\Sounds2012\\NHD_Plus\\"        ##output directory with NHDPlus region geodatabases.
suffix = "albers"                                   ## Suffix to be added to the reprojected nhdflowline feature class.
processList = "E:\\Sounds2012\\NHD_Plus\\Table\\NHDOrderListAllRegions.csv"           ##List of NHDPlus regions to be processed.
projFile = "E:\\SolarPEIS\\ELCA_AOI.gdb\\ELCA_25mile_albers"            ##File with desired projection.

logFileName = workspace + "ReprojectNHDPlus.txt"    ##Logfile Name.
###################################################
# End of Parameters requiring set up.
###################################################

# Import system modules
import sys, string, os, arcgisscripting, math, arcpy, glob
from arcpy import env
from arcpy.sa import *

# Check out any necessary licenses
arcpy.CheckOutExtension("Spatial")
# Set overwrite files to True 
arcpy.geoprocessing.env.overwriteOutput = True
# Set geoprocessing work space
arcpy.geoprocessing.env.workspace = workspace

if os.path.exists(logFileName):
    pass
else:
    logFile = open(logFileName, "w")
    logFile.close()


def timeFun():          #Function to Grab Time
    from datetime import datetime
    b=datetime.now()
    messageTime = b.isoformat()
    return messageTime

#################################################
##Beginning routine to run indivual Fuzzy Viewsheds per development point
#################################################
try:
        
    readFile = open(processList, "r")

    lineCount = 1
    for line in readFile:               #Loop Per Park
        if lineCount == 1:
            lineCount = lineCount + 1
            pass
        else:
            split = line.split(",")
            
            region = str(split[0])
            streamFile = str(split[1])
            orderTable = str(split[2])
            outGdbFull = outWorkspace + region + ".gdb"
            inFile = outGdbFull + "\\nhdflowline" 
            outFile = inFile + "_" + suffix

            lf_read = ""
            logFile = open(logFileName, "r")
            lf_read = logFile.read()
            logFile.close()

            testLine = "Completed " + outFile

            if testLine in lf_read:
                
                messageTime = timeFun()
                logFile = open(logFileName, "a")
                scriptMsg = "Skipping Point - " + outFile + " - has been previously processed" + messageTime
                logFile.write(scriptMsg + "\n")
                logFile.close()
                continue

           
            else: 

                describeAOI = arcpy.Describe(projFile)
                outCS = describeAOI.spatialReference
                env.workspace = outGdbFull             


                arcpy.Project_management(inFile, outFile, outCS)


                messageTime = timeFun()
                logFile = open(logFileName, "a")
                
                scriptMsg = "Completed " + outFile + " - " + messageTime
                logFile.write(scriptMsg + "\n")
                logFile.close()


                lineCount = lineCount + 1
            
except:
    messageTime = timeFun()
    scriptMsg = "SelectByStreamOrder.py Error" 
    print "SelectByStreamOrder.py Exited\nSee log file " + logFileName + " for more details - " + messageTime
    
    
finally:
    messageTime = timeFun()
    scriptMsg = "Select By Attribute " + " << " + messageTime
   
    