###########################################
# StreamSlope.py 
# Draft version Date: May 17, 2011
# Script Written By:  Kirk Sherrill NPS-NRPC-I&M GIS Program
# 
# Abstract: Using stream data and Digital Elevation Model's, this script calculates stream slope per defined Area
# of Analysis (AOA).  Stream slope calculation is derived using only dem cells that are directly coincident with stream
# segments.
##########################################

#######################################
# Start of Parameters requiring set up.
#######################################

## Required Data Layers
workspace = "E:\\Sounds2012\\workspace\\"            ##Scratch workspace

inFile = "E:\\Sounds2012\\NHD_Streams.gdb\\"   ##Workspace with the Stream files
inPrefix = "Streams"         ##Prefix for stream file
demGdb = "E:\\Sounds2012\\DEM_Sounds2012.gdb\\"    ##Workspace with DEM files
prefix = "strSlope_"          ##prefix for output file (will be prefix_{park}) and the feature dataset name as well
outWorkspace = "E:\\Sounds2012\\StreamSlope.gdb\\"    ##Out Geodatabase
parkList = "E:\\Sounds2012\\Sites_CONUS_2012.csv"           ##List of Parks to Process


scratchGdb = "E:\\Sounds2012\\workspace\workGdb.gdb\\"       ##Scratch workspace
logFileName = workspace + "AA_Stream_SlopeSOunds2012.txt"     ##LogFile Name


###################################################
# End of Parameters requiring set up.
###################################################

# Import system modules
import sys, string, os, arcgisscripting, math, arcpy, glob
from arcpy import env
from arcpy.sa import *

# Check out any necessary licenses
arcpy.CheckOutExtension("Spatial")
# Set overwrite files to True 
arcpy.geoprocessing.env.overwriteOutput = True
# Set geoprocessing work space
arcpy.geoprocessing.env.workspace = workspace


def timeFun():          #Function to Grab Time
    from datetime import datetime
    b=datetime.now()
    messageTime = b.isoformat()
    return messageTime

#################################################
##Beginning routine to run indivual Fuzzy Viewsheds per development point
#################################################
try:
       
    readFile = open(parkList, "r")

    if os.path.exists(logFileName):
        pass
    else:
        logFile = open(logFileName, "w")
        logFile.close()
    
    
    lineCount = 1
    for line in readFile:               #Loop Per Park
        if lineCount == 1:
            lineCount = lineCount + 1
            pass
        else:
            split = line.split(",")
            park = str(split[0])
            site = str(split[1])
            siteId = site
            file = inFile + inPrefix + "_" + siteId         #Park Specific file to do the clipping/Masking - Not Dynamic
            
            outFC = prefix  #Output feature class    
            
            outFile = outWorkspace + prefix + "_" + siteId           #Output File

             # Open log file to get a current list of previously-processed Parks
            lf_read = ""
            logFile = open(logFileName, "r")
            lf_read = logFile.read()
            logFile.close()
            
            testLine = "Successfully Processed - " + siteId   #Checking logfile for this syntax - if there then already processed
            if testLine in lf_read:
                
                messageTime = timeFun()
                logFile = open(logFileName, "a")
                scriptMsg = "Skipping - " + siteId + "_has been previously processed" + messageTime
                logFile.write(scriptMsg + "\n")
                logFile.close()
                continue
            else:
                
                ###########
                # Stream Slope per site
                ###########
                
                arcpy.env.workspace = file
                arcpy.env.extent = file
                countFile = arcpy.GetCount_management(file)
                countStr = str(countFile)
                
                
                if int(countStr) != 0:
                    rpjFile = scratchGdb + "rpj_" + siteId    #Temp Reprojected Raster 
                    tempRaster = scratchGdb + "temp_" + siteId    #Temp Stream Raster
                    rcRaster = scratchGdb + "rc_" + siteId        #Temp Reclassed Raster
                    demFile = demGdb + siteId + "_dem"    #dem File
                    outSl = outWorkspace + "slope_" + siteId #Out Stream Slope File
                    cellSize = 10
                    outCS = "E:\\Sounds2012\\DEM_Albers_Sounds2012.gdb\\ACAD001_ACAD_dem"       #Note not Dynamic
                    snapRaster =  demFile           ##Snap Raster
                    arcpy.env.snapRaster = snapRaster 
                    arcpy.env.extent = file
                    arcpy.env.workspace = scratchGdb
                    #Project GCS stream to Albers
                    arcpy.Project_management(file, rpjFile, outCS)
                    arcpy.env.extent = rpjFile
                    
                    #PolyLine to Raster
                    arcpy.PolylineToRaster_conversion(rpjFile, "FType", tempRaster, "MAXIMUM_LENGTH", "NONE", cellSize)
                    #Reclassify Raster to binary (1 Stream) else Nodata
                    outReclass = Reclassify(tempRaster, "Value", RemapRange([[0,1000000,1]]),"NODATA")
                    outReclass.save(rcRaster)
                    demStRc = Raster(rcRaster) * Raster(demFile)
                    outSlope = Slope(demStRc, "Degree")
                    outSlope.save(outSl)

                    
                    arcpy.Delete_management(tempRaster); print "\n" + arcpy.GetMessages() + "\n"
                    arcpy.Delete_management(rpjFile); print "\n" + arcpy.GetMessages() + "\n"
                    arcpy.Delete_management(rcRaster); print "\n" + arcpy.GetMessages() + "\n"
                    arcpy.Delete_management(demStRc); print "\n" + arcpy.GetMessages() + "\n"
   
                else:
                    continue
               
                #Write log message
                            
                logFile = open(logFileName, "a")
                messageTime = timeFun()            
                scriptMsg = "Successfully Processed - " + siteId + " " + messageTime 
                
                logFile.write(scriptMsg + "\n")
                logFile.close()    
                
                lineCount = lineCount + 1
            
except:
    messageTime = timeFun()
    scriptMsg = "Exiting Extract_To_AOI_Arc10.py Error" 
    print "Exiting Extracdt_To_AOI_Arc10.py Exited\nSee log file " + logFileName + " for more details - " + messageTime
    logFile = open(logFileName, "a")
    logFile.write(scriptMsg + "\n")
    logFile.close()
    
finally:
    messageTime = timeFun()
    scriptMsg = "Finished Extract_To_AOI_Arc10 " + " << " + messageTime
    logFile = open(logFileName, "a")
    logFile.write(scriptMsg + "\n")
    logFile.close()
    