###########################################
# SummaryStatisticsWilderness.py
# Draft version Date: December, 2011
# Script Written By:  Kirk Sherrill NPS-NRPC-I&M GIS Program
# 
# Abstract:  Calculated summary statistics for a defined Area of Analysis (AOA)
##########################################

###################################################
# Start of Parameters requiring set up.
###################################################

## Required Data Layers
workspace = "D:\\Sounds2012\\workspace\\"            ##root workspace
inputWorkspace = "D:\\Sounds2012\\Wilderness.gdb\\"  ##Input data and or workspace to be clipped                                    
inputWild   = "Wild*"           ##Wildcard syntax to grab all desired files (Rdd_All*|Rdd_Major*|Rdd_Weighted*)with list raster/feature gp function
aoaData = "D:\\Sounds2012\\AOI_CONUS.gdb\\buf_10mile\\"       ##Workspace with AOA files
aoaSuffix ="10mile"      # #suffix to add for aoi File
outTablePrefix = "Wilderness_10mile_Stats"     ##Prefix for the output gdb appended statistics table.
dataType = "Feature"        ##('Feature'|'Raster') Switch defining if feature class or Raster data is being processed
rasterTable = "No"         ##Define if the raster dataset has a Raster Attribute Table ('yes'|'no')
logFileName = workspace + "Summary_statistics_Wilderness_10m.txt"       ##LogFileName

###################################################
# End of Parameters requiring set up.
###################################################

# Import system modules
import sys, string, os, arcgisscripting, math, arcpy, glob
from arcpy import env
from arcpy.sa import *

# Check out any necessary licenses
arcpy.CheckOutExtension("Spatial")
# Set overwrite files to True 
arcpy.geoprocessing.env.overwriteOutput = True
# Set geoprocessing work space
arcpy.geoprocessing.env.workspace = workspace



def timeFun():          #Function to Grab Time
    from datetime import datetime
    b=datetime.now()
    messageTime = b.isoformat()
    return messageTime

#################################################
##Beginning routine to run indivual Fuzzy Viewsheds per development point
#################################################
try:
   
    env.workspace = inputWorkspace
    if dataType.lower() == "feature":
        dataFiles = arcpy.ListFeatureClasses(inputWild)
    else:
        dataFiles = arcpy.ListRasters(inputWild, "GRID") #Grab the list of folders containing the files needing to be extracted
    
    count = 1   
    for file in dataFiles:          #Data Source Files to be clipped/extracted

        if rasterTable.lower()== "yes" and dataType.lower()!= "feature":
            arcpy.DeleteRasterAttributeTable_management(file) 
            arcpy.BuildRasterAttributeTable_management(file, "Overwrite")
        else:
            pass
                  
        nameSplit = file.split("_")
        if len(nameSplit) == 3:
            siteIdStr = str(nameSplit[1]+ "_" + nameSplit[2])
            siteId = "'" + siteIdStr + "'"
            park = str(nameSplit[2])
        elif len(nameSplit) == 4:
            siteIdStr = str(nameSplit[2] + "_" + nameSplit[3])
            siteId = "'" + siteIdStr + "'"
            park = str(nameSplit[3])
        else:
            break

        if os.path.exists(logFileName):
            pass
        else:
            logFile = open(logFileName, "w")
            logFile.close()

        lf_read = ""
        logFile = open(logFileName, "r")
        lf_read = logFile.read()
        logFile.close()
        
        testLine = "Successfully Processed - " + file    #Checking logfile for this syntax - if there then already processed
        if testLine in lf_read:
            
            messageTime = timeFun()
            logFile = open(logFileName, "a")
            scriptMsg = "Skipping - " + file + "- has been previously processed" + messageTime
            logFile.write(scriptMsg + "\n")
            logFile.close()
            count = count + 1
            continue
        else:
               
            if dataType.lower() == "raster":
                describeAOI = arcpy.Describe(file)   #Describe the AOI DEM
                extentAOI = describeAOI.extent      #AOI Spatial Extent
                asciiExtent = str(extentAOI).split()
                asciiExtentStr = ""  #asciiExtentStr is used in the raster to ascii routine exactly match the input extent from the average file
                        #List to hold the visible fuzzy files
                countAscii = 1
                for cor in asciiExtent:
                    if countAscii <= 4:
                        asciiExtentStr = asciiExtentStr + str(cor) + " "    
                        countAscii = countAscii + 1
                    else:
                        break
                
                arcpy.env.extent = asciiExtentStr
            else:
                arcpy.env.extent = file

                
            aoiFile = aoaData + siteIdStr + "_" + aoaSuffix
            outTable = inputWorkspace + outTablePrefix

            arcpy.env.extent = aoiFile
            '''
            countFile = arcpy.GetCount_management(file)
            countStr = str(countFile)
            if int(countStr) != 0:
                pass
            else:
                continue
            '''
               
            if count == 1:        
                     
                #outTable = inputWorkspace + "\\" + outTablePrefix
                if dataType.lower() == "feature":
                    arcpy.Statistics_analysis(file, outTable, [["Shape_Length","SUM"]])
                    
                else:
                    arcpy.gp.ZonalStatisticsAsTable_sa(aoiFile, "SiteID", file, outTable, "DATA", "ALL") 

                arcpy.DeleteField_management(outTable, ["SiteID"])
                arcpy.AddField_management(outTable, "SITEID", "Text", 45, "", "", "", "", "")
                arcpy.CalculateField_management(outTable, "SITEID", siteId, "PYTHON_9.3")

            else:
                outTableTemp = inputWorkspace + "\\" + siteIdStr + "_temp"
                #outTableTemp = inputWorkspace + "\\" + str(file) + "_temp"

                if dataType.lower() == "feature":
                    arcpy.Statistics_analysis(file, outTableTemp, [["Shape_Length","SUM"]])
                
                else:  #Raster
                    arcpy.gp.ZonalStatisticsAsTable_sa(aoiFile, "SiteID", file, outTableTemp, "DATA", "ALL") 

                arcpy.DeleteField_management(outTableTemp, ["SiteID"])
                arcpy.AddField_management(outTableTemp, "SITEID", "Text", 30, "", "", "", "", "")
                arcpy.CalculateField_management(outTableTemp, "SITEID", siteId, "PYTHON_9.3")

                arcpy.Append_management(outTableTemp, outTable, "NO_TEST","","")
                arcpy.Delete_management(outTableTemp)

            #Write log message
            logFile = open(logFileName, "a")
            messageTime = timeFun()            
            scriptMsg = "Successfully Processed - " + file + " " + messageTime 
            logFile.write(scriptMsg + "\n")
            logFile.close()    
            count = count + 1
       
            
except:
    messageTime = timeFun()
    scriptMsg = "Exiting SummaryStatisticsFlight.py Error" 
    print "Exiting Exiting SummaryStatisticsFlight.py Exited\nSee log file " + logFileName + " for more details - " + messageTime
    logFile = open(logFileName, "a")
    logFile.write(scriptMsg + "\n")
    logFile.close()
    
finally:
    messageTime = timeFun()
    scriptMsg = "Finished SummaryStatisticsFlight.py << " + messageTime
    logFile = open(logFileName, "a")
    logFile.write(scriptMsg + "\n")
    logFile.close()
    