###########################################
# TPI.py
# Draft version Date: Dec 15th, 2011
# Script Written By:  Kirk Sherrill NPS-NRPC-I&M GIS Program
# 
# Abstract:  Derives Topographic Position Index (TPI)using DEM and Slope relationships and neighborhood analysis at
#           the defined Area of Analysis.  TPI logic was obtained from Theobald's LCap Tools - Topographic Position Index (TPI) (2009)
#           1 - Ridge, 2 - Upper Slope, 3 - Middle Slope, 4 - Flat, 5 - Lower Slope, 6 - Valleys.
#
#           Theobald, D.M. 2009. LCAPv1.0: Landscape Connectivity and Pattern tools for ArcGIS
##########################################

###################################################
# Start of Parameters requiring set up.
###################################################

## Required Data Layers
workspace = "D:\\Sounds2012\\workspace\\"            ##Scratch workspace
inGdb = "D:\\Sounds2012\\DEM_Sounds2012.gdb\\"               ##GDB with DEM's
demSuffix = "dem"           #suffix for dem files
outGdb = "D:\\Sounds2012\\TPI_Sounds2012.gdb\\"           ##Geodatabase to house out tables 
workGdb = "D:\\Sounds2012\\workspace\\workGdb.gdb\\"   ##Work Geodatabase for processing

siteList = "D:\\Sounds2012\\Sites_CONUS_2012.csv"           ##List of Sites to Process

logFileName = workspace + "AA_TPI_Sounds2012.txt"


###################################################
# End of Parameters requiring set up.
###################################################

# Import system modules
import sys, string, os, arcgisscripting, math, arcpy, glob
from arcpy import env
from arcpy.sa import *

# Check out any necessary licenses
arcpy.CheckOutExtension("Spatial")
# Set overwrite files to True 
arcpy.geoprocessing.env.overwriteOutput = True


def timeFun():          #Function to Grab Time
    from datetime import datetime
    b=datetime.now()
    messageTime = b.isoformat()
    return messageTime

#################################################
##Beginning routine to run indivual Fuzzy Viewsheds per development point
#################################################
try:
        
    readFile = open(siteList, "r")

    if os.path.exists(logFileName):
        pass
    else:
        logFile = open(logFileName, "w")
        logFile.close()
    
    
    lineCount = 1
    for line in readFile:               #Loop Per Park
        if lineCount == 1:
            lineCount = lineCount + 1
            pass
        else:
            
            split = line.split(",")
            park = str(split[0])
            #site = str(split[1])
            siteId = str(split[1])
            inDem = inGdb + siteId + "_" + demSuffix         #input DEM
            outTPI = outGdb + siteId + "_TPI"   #output TPI
 
             # Open log file to get a current list of previously-processed Parks
            lf_read = ""
            logFile = open(logFileName, "r")
            lf_read = logFile.read()
            logFile.close()
            
            testLine = "Successfully Processed - " + siteId   #Checking logfile for this syntax - if there then already processed
            if testLine in lf_read:
                
                messageTime = timeFun()
                logFile = open(logFileName, "a")
                scriptMsg = "Skipping - " + siteId + "_has been previously processed" + messageTime
                logFile.write(scriptMsg + "\n")
                logFile.close()
                continue
            else:
                arcpy.geoprocessing.env.workspace = inDem
                arcpy.env.cellSize = inDem
                nLat = arcpy.GetRasterProperties_management(inDem, "TOP")
                sLat = arcpy.GetRasterProperties_management(inDem, "BOTTOM")
                wLon = arcpy.GetRasterProperties_management(inDem, "LEFT")
                eLon = arcpy.GetRasterProperties_management(inDem, "RIGHT")
                extentStr = str(wLon) + ", " + str(sLat) + ", " + str(eLon) + ", " + str(nLat)

                arcpy.env.extent = extentStr        #Defined analysis extent
                arcpy.env.cellSize = inDem          #Cellsize
                arcpy.geoprocessing.env.workspace = workGdb #Geoprocessing work environment
                            
                outFocal = workGdb + siteId + "_1"
                outSlope = workGdb + siteId + "_slope"
                diffDem = workGdb + siteId + "_diffDem"
                tpi = outGdb + siteId + "_tpi"
              
                arcpy.gp.FocalStatistics_sa(inDem, outFocal, "Annulus 5 10 CELL", "MEAN", "DATA")
                diffDem1 = (Int((Raster(inDem) - Raster(outFocal) + 0.5)))
                diffDem1.save(diffDem)
                arcpy.gp.Slope_sa(inDem, outSlope, "DEGREE", "1")
                                                
                diffDem = Raster(diffDem)           #Explicitly setting the defined and previously derived diffDem to a raster tag
                outSlope = Raster(outSlope)         #Explicitly setting the defined and previously derived outSlope to a raster tag

                #MapAlgebra syntax to derive the Topographic Position Index
                tpiCalc = Con(diffDem > 20, 1, Con(diffDem > 10, 2, Con(diffDem > -10, Con(outSlope > 5, 3, 4), Con(diffDem > -20, 5, 6))))
                tpiCalc.save(outTPI)
                
                #Write log message
                            
                logFile = open(logFileName, "a")
                messageTime = timeFun()            

                scriptMsg = "Successfully Processed - " + siteId + " " + messageTime 
                logFile.write(scriptMsg + "\n")
                logFile.close()    
                
                lineCount = lineCount + 1
            
except:
    messageTime = timeFun()
    scriptMsg = "Exiting TPI.py Error" 
    print "Exiting TPI.py Exited\nSee log file " + logFileName + " for more details - " + messageTime
    logFile = open(logFileName, "a")
    logFile.write(scriptMsg + "\n")
    logFile.close()
    
finally:
    messageTime = timeFun()
    scriptMsg = "Finished TPI.py " + " << " + messageTime
    logFile = open(logFileName, "a")
    logFile.write(scriptMsg + "\n")
    logFile.close()
    